@@ -191,18 +191,18 @@ View::View( Part *part, QWidget *parent, const char *name)
     if ( !Settings::showQuickFilter() )    
         m_searchBar->hide();
     
-    m_searchCombo->insertItem(i18n("All Articles"));
+    m_searchCombo->insertItem(SmallIcon("exec"), i18n("All Articles"));
     m_searchCombo->insertItem(i18n("New & Unread"));
-    m_searchCombo->insertItem(i18n("New"));
-    m_searchCombo->insertItem(i18n("Unread"));
-    m_searchCombo->insertItem(i18n("Keep Flag Set"));
+    m_searchCombo->insertItem(SmallIcon("kmmsgnew"), i18n("New"));
+    m_searchCombo->insertItem(SmallIcon("kmmsgunseen"), i18n("Unread"));
+    m_searchCombo->insertItem(SmallIcon("kmmsgflag"), i18n("Keep Flag Set"));
 
     QToolTip::add( clearButton, i18n( "Clear filter" ) );
     QToolTip::add( m_searchLine, i18n( "Enter space-separated terms to filter article list" ) );
     QToolTip::add( m_searchCombo, i18n( "Choose what kind of articles to show in article list" ) );
 
     connect(clearButton, SIGNAL( clicked() ),
-                    m_searchLine, SLOT(clear()) );
+                    this, SLOT(slotClearFilter()) );
     connect(m_searchCombo, SIGNAL(activated(int)),
                         this, SLOT(slotSearchComboChanged(int)));
     connect(m_searchLine, SIGNAL(textChanged(const QString &)),
@@ -248,9 +248,6 @@ View::View( Part *part, QWidget *parent, const char *name)
     m_feedSplitter->setSizes( Settings::splitter1Sizes() );
     m_articleSplitter->setSizes( Settings::splitter2Sizes() );
 
-    m_searchCombo->setCurrentItem(Settings::quickFilter());
-    slotSearchComboChanged(Settings::quickFilter());
-
     switch (Settings::viewMode())
     {
         case CombinedView:
@@ -269,6 +266,9 @@ View::View( Part *part, QWidget *parent, const char *name)
     m_tabs->setTitle(i18n("About"), m_mainTab);
     m_displayingAboutPage = true;
     
+    m_searchCombo->setCurrentItem(Settings::quickFilter());
+    slotSearchComboChanged(Settings::quickFilter());
+
     m_fetchTimer=new QTimer(this);
     connect( m_fetchTimer, SIGNAL(timeout()), this, SLOT(slotDoIntervalFetches()) );
     m_fetchTimer->start(1000*60);
@@ -299,6 +299,11 @@ void View::slotOnShutdown()
     m_shuttingDown = true; // prevents slotFrameChanged from crashing
 
     m_transaction->stop();
+    
+    m_tree->setFeedList(0);
+    m_articleViewer->slotShowNode(0);
+    m_articles->slotShowNode(0);
+    
     kdDebug() << "View::slotOnShutdown(): delete feed list" << endl;
     delete m_feedList;
     
@@ -466,8 +471,7 @@ bool View::importFeeds(const QDomDocument& doc)
 
     if (!ok)
     {
-        delete m_feedList;
-        m_feedList = 0;
+        delete feedList;
         return false;
     }
     FeedGroup* fg = new FeedGroup(title);
@@ -839,6 +843,8 @@ void View::slotNodeSelected(TreeNode* node)
     
     m_tabs->showPage(m_mainTab);
 
+    //slotClearFilter();
+
     if (m_viewMode == CombinedView)
         m_articleViewer->slotShowNode(node);
     else
@@ -1021,12 +1027,20 @@ void View::slotNextFeed()
 
 void View::slotNextUnreadArticle()
 {
-    m_articles->slotNextUnreadArticle();
+    TreeNode* sel = m_tree->selectedNode();
+    if (sel && sel->unread() > 0)
+        m_articles->slotNextUnreadArticle();
+    else
+        slotNextUnreadFeed();
 }
 
 void View::slotPrevUnreadArticle()
 {
-    m_articles->slotPreviousUnreadArticle();
+    TreeNode* sel = m_tree->selectedNode();
+    if (sel && sel->unread() > 0)
+        m_articles->slotPreviousUnreadArticle();
+    else
+        slotPrevUnreadFeed();
 }
 
 void View::slotPrevUnreadFeed()
@@ -1337,9 +1351,12 @@ void View::slotSearchComboChanged(int index)
 // from klistviewsearchline
 void View::slotSearchTextChanged(const QString &search)
 {
-    m_queuedSearches++;
-    m_queuedSearch = search;
-    QTimer::singleShot(200, this, SLOT(slotActivateSearch()));
+    if (m_queuedSearch != search)
+    {
+        m_queuedSearches++;
+        m_queuedSearch = search;
+        QTimer::singleShot(200, this, SLOT(slotActivateSearch()));
+    }
 }
 
 void View::slotActivateSearch()
@@ -1549,6 +1566,14 @@ void View::slotMouseOverInfo(const KFileItem *kifi)
     }
 }
 
+void View::slotClearFilter()
+{
+    m_searchLine->clear();
+    m_searchCombo->setCurrentItem(0);
+    slotSearchComboChanged(0);
+    slotSearchTextChanged("");
+}
+
 void View::readProperties(KConfig* config) // this is called when session is being restored
 {
     // read filter settings
