@@ -44,7 +44,6 @@
 #include <kiconloader.h>
 #include <kdialog.h>
 #include <ktextbrowser.h>
-#include <kglobalsettings.h>
 #include <kmimetype.h>
 #include <kmessagebox.h>
 #include <kwin.h>
@@ -78,6 +77,7 @@ using namespace KCal;
 #ifndef WITHOUT_ARTS
 static const char* KMIX_APP_NAME    = "kmix";
 static const char* KMIX_DCOP_OBJECT = "Mixer0";
+static const char* KMIX_DCOP_WINDOW = "kmix-mainwindow#1";
 #endif
 
 // The delay for enabling message window buttons if a zero delay is
@@ -96,7 +96,8 @@ class MessageText : public QTextEdit
 			setWordWrap(QTextEdit::NoWrap);
 		}
 		int scrollBarHeight() const     { return horizontalScrollBar()->height(); }
-		virtual QSize sizeHint() const  { return QSize(contentsWidth(), contentsHeight() + scrollBarHeight()); }
+		int scrollBarWidth() const      { return verticalScrollBar()->width(); }
+		virtual QSize sizeHint() const  { return QSize(contentsWidth() + scrollBarWidth(), contentsHeight() + scrollBarHeight()); }
 };
 
 
@@ -155,6 +156,7 @@ MessageWin::MessageWin(const KAEvent& event, const KAAlarm& alarm, bool reschedu
 	  mDeferButton(0),
 	  mSilenceButton(0),
 	  mDeferDlg(0),
+	  mWinModule(0),
 	  mFlags(event.flags()),
 	  mLateCancel(event.lateCancel()),
 	  mErrorWindow(false),
@@ -167,7 +169,9 @@ MessageWin::MessageWin(const KAEvent& event, const KAAlarm& alarm, bool reschedu
 	  mDeferClosing(false)
 {
 	kdDebug(5950) << "MessageWin::MessageWin(event)" << endl;
-	setAutoSaveSettings(QString::fromLatin1("MessageWin"));     // save window sizes etc.
+	// Set to save settings automatically, but don't save window size.
+	// File alarm window size is saved elsewhere.
+	setAutoSaveSettings(QString::fromLatin1("MessageWin"), false);
 	initView();
 	mWindowList.append(this);
 }
@@ -193,6 +197,7 @@ MessageWin::MessageWin(const KAEvent& event, const DateTime& alarmDateTime, cons
 	  mDeferButton(0),
 	  mSilenceButton(0),
 	  mDeferDlg(0),
+	  mWinModule(0),
 	  mErrorWindow(true),
 	  mNoPostAction(true),
 	  mRecreating(false),
@@ -216,6 +221,7 @@ MessageWin::MessageWin()
 	  mPlayObject(0),
 	  mSilenceButton(0),
 	  mDeferDlg(0),
+	  mWinModule(0),
 	  mErrorWindow(false),
 	  mNoPostAction(true),
 	  mRecreating(false),
@@ -235,6 +241,8 @@ MessageWin::~MessageWin()
 {
 	kdDebug(5950) << "MessageWin::~MessageWin()\n";
 	stopPlay();
+	delete mWinModule;
+	mWinModule = 0;
 	for (MessageWin* w = mWindowList.first();  w;  w = mWindowList.next())
 		if (w == this)
 		{
@@ -355,17 +363,21 @@ void MessageWin::initView()
 				text->setPaper(mBgColour);
 				text->setPaletteForegroundColor(mFgColour);
 				text->setFont(mFont);
-				int h = text->sizeHint().height();
-				text->setMinimumHeight(h - text->scrollBarHeight());
-				text->setMaximumHeight(h);
-				QWhatsThis::add(text, i18n("The alarm message"));
 				int lineSpacing = text->fontMetrics().lineSpacing();
+				QSize s = text->sizeHint();
+				int h = s.height();
+				text->setMaximumHeight(h + text->scrollBarHeight());
+				text->setMinimumHeight(QMIN(h, lineSpacing*4));
+				text->setMaximumWidth(s.width() + text->scrollBarWidth());
+				QWhatsThis::add(text, i18n("The alarm message"));
 				int vspace = lineSpacing/2 - marginKDE2;
 				int hspace = lineSpacing - marginKDE2 - KDialog::marginHint();
 				topLayout->addSpacing(vspace);
 				topLayout->addStretch();
 				// Don't include any horizontal margins if message is 2/3 screen width
-				if (text->sizeHint().width() >= KWinModule(0, KWinModule::INFO_DESKTOP).workArea().width()*2/3)
+				if (!mWinModule)
+					mWinModule = new KWinModule(0, KWinModule::INFO_DESKTOP);
+				if (text->sizeHint().width() >= mWinModule->workArea().width()*2/3)
 					topLayout->addWidget(text, 1, Qt::AlignHCenter);
 				else
 				{
@@ -496,7 +508,7 @@ void MessageWin::initView()
 	}
 
 #ifndef WITHOUT_ARTS
-	if (!mAudioFile.isEmpty())
+	if (!mAudioFile.isEmpty()  &&  mVolume)
 	{
 		// Silence button to stop sound repetition
 		QPixmap pixmap = MainBarIcon("player_stop");
@@ -685,6 +697,8 @@ void MessageWin::playAudio()
 	}
 	if (!mAudioFile.isEmpty())
 	{
+		if (!mVolume)
+			return;    // ensure zero volume doesn't play anything
 #ifdef WITHOUT_ARTS
 		QString play = mAudioFile;
 		QString file = QString::fromLatin1("file:");
@@ -727,7 +741,7 @@ void MessageWin::slotPlayAudio()
 		if (!mPlayObject->object().isNull())
 			checkAudioPlay();
 #if KDE_VERSION >= 308
-		if (!mUsingKMix)
+		if (!mUsingKMix  &&  mVolume >= 0)
 		{
 			// Output error message now that everything else has been done.
 			// (Outputting it earlier would delay things until it is acknowledged.)
@@ -755,25 +769,31 @@ void MessageWin::initAudio(bool firstTime)
 	mPlayObject = factory.createPlayObject(mLocalAudioFile, true);
 	if (firstTime)
 	{
-		// Get the current master volume from KMix
-		int vol = getKMixVolume();
-		if (vol >= 0)
+		// Save the existing sound volume setting for restoration afterwards,
+		// and set the desired volume for the alarm.
+		mUsingKMix = false;
+		if (mVolume >= 0)
 		{
-			mOldVolume = vol;    // success
-			mUsingKMix = true;
+			// Get the current master volume from KMix
+			int vol = getKMixVolume();
+			if (vol >= 0)
+			{
+				mOldVolume = vol;    // success
+				mUsingKMix = true;
+				setKMixVolume(static_cast<int>(mVolume * 100));
+			}
 		}
-		else
+		if (!mUsingKMix)
 		{
-			// Can't use KMix to set the master volume, so just adjust
-			// within the current master volume.
+			/* Adjust within the current master volume, because either
+			 * a) volume is not specified, in which case we want to play at
+			 *    100% of the current master volume setting, or
+			 * b) KMix is not available to set the master volume.
+			 */
 			mOldVolume = sserver.outVolume().scaleFactor();    // save volume for restoration afterwards
-			mUsingKMix = false;
+			sserver.outVolume().scaleFactor(mVolume >= 0 ? mVolume : 1);
 		}
 	}
-	if (!mUsingKMix)
-		sserver.outVolume().scaleFactor(mVolume >= 0 ? mVolume : 1);
-	else if (mVolume >= 0)
-		setKMixVolume(static_cast<int>(mVolume * 100));
 	mSilenceButton->setEnabled(true);
 	mPlayed = false;
 	connect(mPlayObject, SIGNAL(playObjectCreated()), SLOT(checkAudioPlay()));
@@ -797,12 +817,17 @@ void MessageWin::checkAudioPlay()
 	{
 		// The file has loaded and is ready to play, or play has completed
 		if (mPlayedOnce  &&  !mAudioRepeat)
+		{
+			// Play has completed
+			stopPlay();
 			return;
+		}
 
 		// Start playing the file, either for the first time or again
 		kdDebug(5950) << "MessageWin::checkAudioPlay(): start\n";
 		if (!mPlayedOnce)
 		{
+			// Start playing the file for the first time
 			mAudioFileLoadSecs = mAudioFileLoadStart.secsTo(QTime::currentTime());
 			if (mAudioFileLoadSecs < 0)
 				mAudioFileLoadSecs += 86400;
@@ -850,8 +875,8 @@ void MessageWin::checkAudioPlay()
 }
 
 /******************************************************************************
-*  Called when the Silence button is clicked.
-*  Stops playing the sound file.
+*  Called when play completes, the Silence button is clicked, or the window is
+*  closed, to reset the sound volume and terminate audio access.
 */
 void MessageWin::stopPlay()
 {
@@ -860,7 +885,15 @@ void MessageWin::stopPlay()
 	{
 		// Restore the sound volume to what it was before the sound file
 		// was played, provided that nothing else has modified it since.
-		if (!mUsingKMix)
+		if (mUsingKMix)
+		{
+			int eventVolume = static_cast<int>(mVolume * 100);
+			int currentVolume = getKMixVolume();
+			// Volume returned isn't always exactly equal to volume set
+			if (currentVolume < 0  ||  abs(currentVolume - eventVolume) < 5)
+				setKMixVolume(static_cast<int>(mOldVolume));
+		}
+		else
 		{
 			KArtsServer aserver;
 			Arts::StereoVolumeControl svc = aserver.server().outVolume();
@@ -871,19 +904,14 @@ void MessageWin::stopPlay()
 			if (currentVolume == eventVolume)
 				svc.scaleFactor(mOldVolume);
 		}
-		else if (mVolume >= 0)
-		{
-			int eventVolume = static_cast<int>(mVolume * 100);
-			int currentVolume = getKMixVolume();
-			// Volume returned isn't always exactly equal to volume set
-			if (currentVolume < 0  ||  abs(currentVolume - eventVolume) < 5)
-				setKMixVolume(static_cast<int>(mOldVolume));
-		}
 	}
 	delete mPlayObject;      mPlayObject = 0;
 	delete mArtsDispatcher;  mArtsDispatcher = 0;
 	if (!mLocalAudioFile.isEmpty())
+	{
 		KIO::NetAccess::removeTempFile(mLocalAudioFile);   // removes it only if it IS a temporary file
+		mLocalAudioFile = QString::null;
+	}
 	if (mSilenceButton)
 		mSilenceButton->setEnabled(false);
 #endif
@@ -939,6 +967,10 @@ bool MessageWin::runKMix()
 		// KMix is not already running, so start it
 		if (KApplication::startServiceByDesktopName(QString::fromLatin1(KMIX_APP_NAME), QString::null, &mKMixError, &mKMixName))
 			return false;
+		// Minimise its window - don't use hide() since this would remove all
+		// trace of it from the panel if it is not configured to be docked in
+		// the system tray.
+		kapp->dcopClient()->send(mKMixName, KMIX_DCOP_WINDOW, "minimize()", QString::null);
 	}
 	return true;
 }
@@ -986,6 +1018,25 @@ void MessageWin::show()
 	MainWindowBase::show();
 }
 
+/******************************************************************************
+*  Returns the window's recommended size exclusive of its frame.
+*  For message windows, the size if limited to fit inside the working area of
+*  the desktop.
+*/
+QSize MessageWin::sizeHint() const
+{
+	if (mAction != KAEvent::MESSAGE)
+		return MainWindowBase::sizeHint();
+	if (!mWinModule)
+		mWinModule = new KWinModule(0, KWinModule::INFO_DESKTOP);
+	QSize frame = frameGeometry().size();
+	QSize contents = geometry().size();
+	QSize desktop  = mWinModule->workArea().size();
+	QSize maxSize(desktop.width() - (frame.width() - contents.width()),
+	              desktop.height() - (frame.height() - contents.height()));
+	return MainWindowBase::sizeHint().boundedTo(maxSize);
+}
+
 /******************************************************************************
 *  Called when the window is shown.
 *  The first time, output any required audio notification, and reschedule or
@@ -1000,6 +1051,11 @@ void MessageWin::showEvent(QShowEvent* se)
 			enableButtons();    // don't bother repositioning error messages
 		else
 		{
+			/* Set the window size.
+			 * Note that the frame thickness is not yet known when this
+			 * method is called, so for large windows the size needs to be
+			 * set again later.
+			 */
 			QSize s = sizeHint();     // fit the window round the message
 			if (mAction == KAEvent::FILE  &&  !mErrorMsgs.count())
 				KAlarm::readConfigWindowSize("FileMessage", s);
@@ -1018,7 +1074,10 @@ void MessageWin::showEvent(QShowEvent* se)
 				 *      window in showEvent(), a flicker is unavoidable.
 				 *      See the Qt documentation on window geometry for more details.
 				 */
-				QRect desk = KGlobalSettings::desktopGeometry(this);
+				// PROBLEM: The frame size is not known yet!
+				if (!mWinModule)
+					mWinModule = new KWinModule(0, KWinModule::INFO_DESKTOP);
+				QRect desk = mWinModule->workArea();
 				QPoint cursor = QCursor::pos();
 				QRect frame = frameGeometry();
 				QRect rect  = geometry();
@@ -1053,6 +1112,11 @@ void MessageWin::showEvent(QShowEvent* se)
 			}
 			if (!mPositioning)
 				displayComplete();    // play audio, etc.
+			if (mAction == KAEvent::MESSAGE)
+			{
+				// Set the window size once the frame size is known
+				QTimer::singleShot(0, this, SLOT(setMaxSize()));
+			}
 		}
 		mShown = true;
 	}
@@ -1072,6 +1136,16 @@ void MessageWin::moveEvent(QMoveEvent* e)
 	}
 }
 
+/******************************************************************************
+*  Reset the iniital window size if it exceeds the working area of the desktop.
+*/
+void MessageWin::setMaxSize()
+{
+	QSize s = sizeHint();
+	if (width() > s.width()  ||  height() > s.height())
+		resize(s);
+}
+
 /******************************************************************************
 *  Called when the window has been displayed properly (in its correct position),
 *  to play sounds and reschedule the event.
@@ -1107,6 +1181,7 @@ void MessageWin::resizeEvent(QResizeEvent* re)
 {
 	if (mRestoreHeight)
 	{
+		// Restore the window height on session restoration
 		if (mRestoreHeight != re->size().height())
 		{
 			QSize size = re->size();
