@@ -1,7 +1,7 @@
 /*
  *  prefdlg.cpp  -  program preferences dialog
  *  Program:  kalarm
- *  (C) 2001 - 2005 by David Jarvie <software@astrojar.org.uk>
+ *  Copyright (c) 2001 - 2005 by David Jarvie <software@astrojar.org.uk>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -13,9 +13,9 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "kalarm.h"
@@ -253,6 +253,9 @@ MiscPrefTab::MiscPrefTab(QVBox* frame)
 
 	mAutostartTrayIcon1 = new QCheckBox(i18n("Autostart at &login"), group, "autoTray");
 	mAutostartTrayIcon1->setFixedSize(mAutostartTrayIcon1->sizeHint());
+#ifdef AUTOSTART_BY_KALARMD
+	connect(mAutostartTrayIcon1, SIGNAL(toggled(bool)), SLOT(slotAutostartToggled(bool)));
+#endif
 	QWhatsThis::add(mAutostartTrayIcon1,
 	      i18n("Check to run %1 whenever you start KDE.").arg(progname));
 	grid->addMultiCellWidget(mAutostartTrayIcon1, row, row, 1, 2, alignment);
@@ -288,6 +291,9 @@ MiscPrefTab::MiscPrefTab(QVBox* frame)
 
 	mAutostartTrayIcon2 = new QCheckBox(i18n("Autostart system tray &icon at login"), group, "autoRun");
 	mAutostartTrayIcon2->setFixedSize(mAutostartTrayIcon2->sizeHint());
+#ifdef AUTOSTART_BY_KALARMD
+	connect(mAutostartTrayIcon2, SIGNAL(toggled(bool)), SLOT(slotAutostartToggled(bool)));
+#endif
 	QWhatsThis::add(mAutostartTrayIcon2,
 	      i18n("Check to display the system tray icon whenever you start KDE."));
 	grid->addMultiCellWidget(mAutostartTrayIcon2, row, row, 1, 2, alignment);
@@ -442,10 +448,13 @@ void MiscPrefTab::restore()
 	setExpiredControls(preferences->mExpiredKeepDays);
 	QString xtermCmd = preferences->cmdXTermCommand();
 	int id = 0;
-	for ( ;  id < mXtermCount;  ++id)
+	if (!xtermCmd.isEmpty())
 	{
-		if (mXtermType->find(id)  &&  xtermCmd == xtermCommands[id])
-			break;
+		for ( ;  id < mXtermCount;  ++id)
+		{
+			if (mXtermType->find(id)  &&  xtermCmd == xtermCommands[id])
+				break;
+		}
 	}
 	mXtermType->setButton(id);
 	mXtermCommand->setEnabled(id == mXtermCount);
@@ -455,14 +464,39 @@ void MiscPrefTab::restore()
 
 void MiscPrefTab::apply(bool syncToDisc)
 {
+	// First validate anything entered in Other X-terminal command
+	int xtermID = mXtermType->selectedId();
+	if (xtermID >= mXtermCount)
+	{
+		QString cmd = mXtermCommand->text();
+		if (cmd.isEmpty())
+			xtermID = 0;       // 'Other' is only acceptable if it's non-blank
+		else
+		{
+			int i = cmd.find(' ');    // find the end of the terminal window name
+			if (i > 0)
+				cmd = cmd.left(i);
+			if (KStandardDirs::findExe(cmd).isEmpty())
+			{
+				mXtermCommand->setFocus();
+				if (KMessageBox::warningContinueCancel(this, i18n("Command to invoke terminal window not found:\n%1").arg(cmd))
+					        != KMessageBox::Continue)
+					return;
+			}
+		}
+	}
 	Preferences* preferences = Preferences::instance();
-	preferences->mAutostartDaemon = mAutostartDaemon->isChecked();
 	bool systray = mRunInSystemTray->isChecked();
 	preferences->mRunInSystemTray        = systray;
 	preferences->mDisableAlarmsIfStopped = mDisableAlarmsIfStopped->isChecked();
 	if (mQuitWarn->isEnabled())
 		preferences->setQuitWarn(mQuitWarn->isChecked());
 	preferences->mAutostartTrayIcon = systray ? mAutostartTrayIcon1->isChecked() : mAutostartTrayIcon2->isChecked();
+#ifdef AUTOSTART_BY_KALARMD
+	preferences->mAutostartDaemon = mAutostartDaemon->isChecked() || preferences->mAutostartTrayIcon;
+#else
+	preferences->mAutostartDaemon = mAutostartDaemon->isChecked();
+#endif
 	preferences->setConfirmAlarmDeletion(mConfirmAlarmDeletion->isChecked());
 	int sod = mStartOfDay->value();
 	preferences->mStartOfDay.setHMS(sod/60, sod%60, 0);
@@ -470,8 +504,7 @@ void MiscPrefTab::apply(bool syncToDisc)
 	preferences->mFeb29RecurType  = (feb29 >= 0) ? Preferences::Feb29Type(feb29) : Preferences::default_feb29RecurType;
 	preferences->mExpiredKeepDays = !mKeepExpired->isChecked() ? 0
 	                              : mPurgeExpired->isChecked() ? mPurgeAfter->value() : -1;
-	int id = mXtermType->selectedId();
-	preferences->mCmdXTermCommand = (id < mXtermCount) ? xtermCommands[id] : mXtermCommand->text();
+	preferences->mCmdXTermCommand = (xtermID < mXtermCount) ? xtermCommands[xtermID] : mXtermCommand->text();
 	PrefsTabBase::apply(syncToDisc);
 }
 
@@ -513,6 +546,18 @@ void MiscPrefTab::slotRunModeToggled(bool)
 	mDisableAlarmsIfStopped->setEnabled(systray);
 }
 
+/******************************************************************************
+* If autostart at login is selected, the daemon must be autostarted so that it
+* can autostart KAlarm, in which case disable the daemon autostart option.
+*/
+void MiscPrefTab::slotAutostartToggled(bool)
+{
+#ifdef AUTOSTART_BY_KALARMD
+	bool autostart = mRunInSystemTray->isChecked() ? mAutostartTrayIcon1->isChecked() : mAutostartTrayIcon2->isChecked();
+	mAutostartDaemon->setEnabled(!autostart);
+#endif
+}
+
 void MiscPrefTab::slotDisableIfStoppedToggled(bool)
 {
 	bool disable = (mDisableAlarmsIfStopped->isChecked());
@@ -878,25 +923,17 @@ EditPrefTab::EditPrefTab(QVBox* frame)
 	QBoxLayout* layout = new QVBoxLayout(group, marginKDE2 + KDialog::marginHint(), KDialog::spacingHint());
 	layout->addSpacing(groupTopMargin);
 
-	QHBox* box = new QHBox(group);
-	box->setSpacing(KDialog::spacingHint());
-	layout->addWidget(box);
-	mDefaultLateCancel = new QCheckBox(LateCancelSelector::i18n_n_CancelIfLate(), box, "defCancelLate");
-	mDefaultLateCancel->setMinimumSize(mDefaultLateCancel->sizeHint());
-	QWhatsThis::add(mDefaultLateCancel, defsetting.arg(LateCancelSelector::i18n_CancelIfLate()));
-	box->setStretchFactor(new QWidget(box), 1);    // left adjust the control
-
-	mDefaultAutoClose = new QCheckBox(LateCancelSelector::i18n_i_AutoCloseWinLC(), box, "defAutoClose");
-	mDefaultAutoClose->setMinimumSize(mDefaultAutoClose->sizeHint());
-	QWhatsThis::add(mDefaultAutoClose, defsetting.arg(LateCancelSelector::i18n_AutoCloseWin()));
-	box->setFixedHeight(box->sizeHint().height());
-
 	mDefaultConfirmAck = new QCheckBox(EditAlarmDlg::i18n_k_ConfirmAck(), group, "defConfAck");
 	mDefaultConfirmAck->setMinimumSize(mDefaultConfirmAck->sizeHint());
 	QWhatsThis::add(mDefaultConfirmAck, defsetting.arg(EditAlarmDlg::i18n_ConfirmAck()));
 	layout->addWidget(mDefaultConfirmAck, 0, Qt::AlignAuto);
 
-	box = new QHBox(group);
+	mDefaultAutoClose = new QCheckBox(LateCancelSelector::i18n_i_AutoCloseWinLC(), group, "defAutoClose");
+	mDefaultAutoClose->setMinimumSize(mDefaultAutoClose->sizeHint());
+	QWhatsThis::add(mDefaultAutoClose, defsetting.arg(LateCancelSelector::i18n_AutoCloseWin()));
+	layout->addWidget(mDefaultAutoClose, 0, Qt::AlignAuto);
+
+	QHBox* box = new QHBox(group);
 	box->setSpacing(KDialog::spacingHint());
 	layout->addWidget(box);
 	QLabel* label = new QLabel(i18n("Reminder &units:"), box);
@@ -916,25 +953,23 @@ EditPrefTab::EditPrefTab(QVBox* frame)
 	layout->addWidget(mDefaultSpecialActions);
 
 	// SOUND
-	group = new QButtonGroup(SoundPicker::i18n_Sound(), mPage, "soundGroup");
-	QGridLayout* grid = new QGridLayout(group, 4, 3, marginKDE2 + KDialog::marginHint(), KDialog::spacingHint());
-	grid->setColStretch(2, 1);
-	grid->addColSpacing(0, indentWidth());
-	grid->addColSpacing(1, indentWidth());
-	grid->addRowSpacing(0, fontMetrics().lineSpacing()/2);
+	QButtonGroup* bgroup = new QButtonGroup(SoundPicker::i18n_Sound(), mPage, "soundGroup");
+	layout = new QVBoxLayout(bgroup, marginKDE2 + KDialog::marginHint(), KDialog::spacingHint());
+	layout->addSpacing(groupTopMargin);
 
-	mDefaultSound = new QCheckBox(SoundPicker::i18n_s_Sound(), group, "defSound");
+	box = new QHBox(bgroup);
+	box->setSpacing(KDialog::spacingHint());
+	layout->addWidget(box);
+	mDefaultSound = new QCheckBox(SoundPicker::i18n_s_Sound(), box, "defSound");
 	mDefaultSound->setMinimumSize(mDefaultSound->sizeHint());
 	QWhatsThis::add(mDefaultSound, defsetting.arg(SoundPicker::i18n_Sound()));
-	grid->addMultiCellWidget(mDefaultSound, 1, 1, 0, 2, Qt::AlignAuto);
 
-	mDefaultBeep = new QCheckBox(i18n("&Beep"), group, "defBeep");
+	mDefaultBeep = new QCheckBox(i18n("&Beep"), box, "defBeep");
 	mDefaultBeep->setMinimumSize(mDefaultBeep->sizeHint());
 	QWhatsThis::add(mDefaultBeep,
 	      i18n("Check to select Beep as the default setting for \"%1\" in the alarm edit dialog.").arg(SoundPicker::i18n_Sound()));
-	grid->addMultiCellWidget(mDefaultBeep, 2, 2, 1, 2, Qt::AlignAuto);
 
-	box = new QHBox(group);   // this is to control the QWhatsThis text display area
+	box = new QHBox(bgroup);   // this is to control the QWhatsThis text display area
 	box->setSpacing(KDialog::spacingHint());
 	mDefaultSoundFileLabel = new QLabel(i18n("Sound &file:"), box);
 	mDefaultSoundFileLabel->setFixedSize(mDefaultSoundFileLabel->sizeHint());
@@ -948,16 +983,16 @@ EditPrefTab::EditPrefTab(QVBox* frame)
 	QWhatsThis::add(box,
 	      i18n("Enter the default sound file to use in the alarm edit dialog."));
 	box->setFixedHeight(box->sizeHint().height());
-	grid->addMultiCellWidget(box, 3, 3, 1, 2);
+	layout->addWidget(box);
 
-	mDefaultSoundRepeat = new QCheckBox(i18n("Repea&t sound file"), group, "defRepeatSound");
+	mDefaultSoundRepeat = new QCheckBox(i18n("Repea&t sound file"), bgroup, "defRepeatSound");
 	mDefaultSoundRepeat->setMinimumSize(mDefaultSoundRepeat->sizeHint());
 	QWhatsThis::add(mDefaultSoundRepeat, i18n("sound file \"Repeat\" checkbox", "The default setting for sound file \"%1\" in the alarm edit dialog.").arg(SoundPicker::i18n_Repeat()));
-	grid->addWidget(mDefaultSoundRepeat, 4, 2, Qt::AlignAuto);
+	layout->addWidget(mDefaultSoundRepeat, 0, Qt::AlignAuto);
 #ifdef WITHOUT_ARTS
 	mDefaultSoundRepeat->hide();
 #endif
-	group->setFixedHeight(group->sizeHint().height());
+	bgroup->setFixedHeight(bgroup->sizeHint().height());
 
 	// COMMAND ALARMS
 	group = new QGroupBox(i18n("Command Alarms"), mPage);
@@ -987,13 +1022,20 @@ EditPrefTab::EditPrefTab(QVBox* frame)
 	QWhatsThis::add(mDefaultEmailBcc, defsetting.arg(EditAlarmDlg::i18n_CopyEmailToSelf()));
 	layout->addWidget(mDefaultEmailBcc, 0, Qt::AlignAuto);
 
-	// RECURRENCE
-	QHBox* itemBox = new QHBox(mPage);   // this is to control the QWhatsThis text display area
-	box = new QHBox(itemBox);
+	// Late cancellation
+	box = new QHBox(mPage);
 	box->setSpacing(KDialog::spacingHint());
-	label = new QLabel(i18n("&Recurrence:"), box);
+	mDefaultLateCancel = new QCheckBox(LateCancelSelector::i18n_n_CancelIfLate(), box, "defCancelLate");
+	mDefaultLateCancel->setMinimumSize(mDefaultLateCancel->sizeHint());
+	QWhatsThis::add(mDefaultLateCancel, defsetting.arg(LateCancelSelector::i18n_CancelIfLate()));
+	box->setStretchFactor(new QWidget(box), 1);    // left adjust the control
+
+	// RECURRENCE
+	QHBox* itemBox = new QHBox(box);   // this is to control the QWhatsThis text display area
+	itemBox->setSpacing(KDialog::spacingHint());
+	label = new QLabel(i18n("&Recurrence:"), itemBox);
 	label->setFixedSize(label->sizeHint());
-	mDefaultRecurPeriod = new QComboBox(box, "defRecur");
+	mDefaultRecurPeriod = new QComboBox(itemBox, "defRecur");
 	mDefaultRecurPeriod->insertItem(RecurrenceEdit::i18n_NoRecur());
 	mDefaultRecurPeriod->insertItem(RecurrenceEdit::i18n_AtLogin());
 	mDefaultRecurPeriod->insertItem(RecurrenceEdit::i18n_HourlyMinutely());
@@ -1003,10 +1045,9 @@ EditPrefTab::EditPrefTab(QVBox* frame)
 	mDefaultRecurPeriod->insertItem(RecurrenceEdit::i18n_Yearly());
 	mDefaultRecurPeriod->setFixedSize(mDefaultRecurPeriod->sizeHint());
 	label->setBuddy(mDefaultRecurPeriod);
-	QWhatsThis::add(box,
+	QWhatsThis::add(itemBox,
 	      i18n("The default setting for the recurrence rule in the alarm edit dialog."));
-	itemBox->setStretchFactor(new QWidget(itemBox), 1);
-	itemBox->setFixedHeight(box->sizeHint().height());
+	box->setFixedHeight(itemBox->sizeHint().height());
 
 	mPage->setStretchFactor(new QWidget(mPage), 1);    // top adjust the widgets
 }
@@ -1017,6 +1058,7 @@ void EditPrefTab::restore()
 	mDefaultLateCancel->setChecked(preferences->mDefaultLateCancel);
 	mDefaultAutoClose->setChecked(preferences->mDefaultAutoClose);
 	mDefaultConfirmAck->setChecked(preferences->mDefaultConfirmAck);
+	mDefaultSound->setChecked(preferences->mDefaultSound);
 	mDefaultBeep->setChecked(preferences->mDefaultBeep);
 	mDefaultSoundFile->setText(preferences->mDefaultSoundFile);
 	mDefaultSoundRepeat->setChecked(preferences->mDefaultSoundRepeat);
@@ -1034,6 +1076,7 @@ void EditPrefTab::apply(bool syncToDisc)
 	preferences->mDefaultLateCancel  = mDefaultLateCancel->isChecked() ? 1 : 0;
 	preferences->mDefaultAutoClose   = mDefaultAutoClose->isChecked();
 	preferences->mDefaultConfirmAck  = mDefaultConfirmAck->isChecked();
+	preferences->mDefaultSound       = mDefaultSound->isChecked();
 	preferences->mDefaultBeep        = mDefaultBeep->isChecked();
 	preferences->mDefaultSoundFile   = mDefaultSoundFile->text();
 	preferences->mDefaultSoundRepeat = mDefaultSoundRepeat->isChecked();
@@ -1062,6 +1105,7 @@ void EditPrefTab::setDefaults()
 	mDefaultLateCancel->setChecked(Preferences::default_defaultLateCancel);
 	mDefaultAutoClose->setChecked(Preferences::default_defaultAutoClose);
 	mDefaultConfirmAck->setChecked(Preferences::default_defaultConfirmAck);
+	mDefaultSound->setChecked(Preferences::default_defaultSound);
 	mDefaultBeep->setChecked(Preferences::default_defaultBeep);
 	mDefaultSoundFile->setText(Preferences::default_defaultSoundFile);
 	mDefaultSoundRepeat->setChecked(Preferences::default_defaultSoundRepeat);
