@@ -22,6 +22,10 @@
  *
  *********************************************************************/
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "rfcdecoder.h"
 
 #include "imapparser.h"
@@ -37,6 +41,12 @@
 #include <stdlib.h>
 #include <unistd.h>
 
+#ifdef HAVE_LIBSASL2
+extern "C" {
+#include <sasl/sasl.h>
+}
+#endif
+
 #include <qregexp.h>
 #include <qbuffer.h>
 #include <qstring.h>
@@ -45,7 +55,8 @@
 #include <kdebug.h>
 #include <kmdcodec.h>
 #include <kurl.h>
-#include <kio/kdesasl.h>
+#include <kasciistricmp.h>
+#include <kasciistringtools.h>
 
 imapParser::imapParser ()
 {
@@ -53,7 +64,7 @@ imapParser::imapParser ()
   completeQueue.setAutoDelete (true);
   currentState = ISTATE_NO;
   commandCounter = 0;
-  lastHandled = NULL;
+  lastHandled = 0;
 }
 
 imapParser::~imapParser ()
@@ -100,7 +111,9 @@ imapParser::sendCommand (imapCommand * aCmd)
   else if (command.find ("SEARCH") != -1
            || command == "GETACL"
            || command == "LISTRIGHTS"
-           || command == "MYRIGHTS")
+           || command == "MYRIGHTS"
+           || command == "GETANNOTATION"
+           || command == "NAMESPACE")
   {
     lastResults.clear ();
   }
@@ -136,42 +149,154 @@ imapParser::clientLogin (const QString & aUser, const QString & aPass,
   return retVal;
 }
 
+#ifdef HAVE_LIBSASL2
+static bool sasl_interact( KIO::SlaveBase *slave, KIO::AuthInfo &ai, void *in )
+{
+  kdDebug(7116) << "sasl_interact" << endl;
+  sasl_interact_t *interact = ( sasl_interact_t * ) in;
+
+  //some mechanisms do not require username && pass, so it doesn't need a popup
+  //window for getting this info
+  for ( ; interact->id != SASL_CB_LIST_END; interact++ ) {
+    if ( interact->id == SASL_CB_AUTHNAME ||
+         interact->id == SASL_CB_PASS ) {
+
+      if ( ai.username.isEmpty() || ai.password.isEmpty() ) {
+        if (!slave->openPassDlg(ai))
+          return false;
+      }
+      break;
+    }
+  }
+
+  interact = ( sasl_interact_t * ) in;
+  while( interact->id != SASL_CB_LIST_END ) {
+    kdDebug(7116) << "SASL_INTERACT id: " << interact->id << endl;
+    switch( interact->id ) {
+      case SASL_CB_USER:
+      case SASL_CB_AUTHNAME:
+        kdDebug(7116) << "SASL_CB_[USER|AUTHNAME]: '" << ai.username << "'" << endl;
+        interact->result = strdup( ai.username.utf8() );
+        interact->len = strlen( (const char *) interact->result );
+        break;
+      case SASL_CB_PASS:
+        kdDebug(7116) << "SASL_CB_PASS: [hidden] " << endl;
+        interact->result = strdup( ai.password.utf8() );
+        interact->len = strlen( (const char *) interact->result );
+        break;
+      default:
+        interact->result = 0; interact->len = 0;
+        break;
+    }
+    interact++;
+  }
+  return true;
+}
+#endif
 
 bool
-imapParser::clientAuthenticate (const QString & aUser, const QString & aPass,
-  const QString & aAuth, bool isSSL, QString & resultInfo)
+imapParser::clientAuthenticate ( KIO::SlaveBase *slave, KIO::AuthInfo &ai,
+  const QString & aFQDN, const QString & aAuth, bool isSSL, QString & resultInfo)
 {
-  imapCommand *cmd;
   bool retVal = false;
+#ifdef HAVE_LIBSASL2
+  int result;
+  sasl_conn_t *conn = 0;
+  sasl_interact_t *client_interact = 0;
+  const char *out = 0;
+  uint outlen = 0;
+  const char *mechusing = 0;
+  QByteArray tmp, challenge;
+
+  kdDebug(7116) << "aAuth: " << aAuth << " FQDN: " << aFQDN << " isSSL: " << isSSL << endl;
 
   // see if server supports this authenticator
   if (!hasCapability ("AUTH=" + aAuth))
     return false;
 
+//  result = sasl_client_new( isSSL ? "imaps" : "imap",
+  result = sasl_client_new( "imap", /* FIXME: with cyrus-imapd, even imaps' digest-uri 
+                                       must be 'imap'. I don't know if it's good or bad. */
+                       aFQDN.latin1(),
+                       0, 0, 0, 0, &conn );
+
+  if ( result != SASL_OK ) {
+    kdDebug(7116) << "sasl_client_new failed with: " << result << endl;
+    resultInfo = QString::fromUtf8( sasl_errdetail( conn ) );
+    return false;
+  }
+
+  do {
+    result = sasl_client_start(conn, aAuth.latin1(), &client_interact,
+                       hasCapability("SASL-IR") ? &out : 0, &outlen, &mechusing);
+
+    if ( result == SASL_INTERACT )
+      if ( !sasl_interact( slave, ai, client_interact ) ) {
+        sasl_dispose( &conn );
+        return false;
+      }
+  } while ( result == SASL_INTERACT );
+
+  if ( result != SASL_CONTINUE && result != SASL_OK ) {
+    kdDebug(7116) << "sasl_client_start failed with: " << result << endl;
+    resultInfo = QString::fromUtf8( sasl_errdetail( conn ) );
+    sasl_dispose( &conn );
+    return false;
+  }
+  imapCommand *cmd;
+
+  tmp.setRawData( out, outlen );
+  KCodecs::base64Encode( tmp, challenge );
+  tmp.resetRawData( out, outlen );
   // then lets try it
-  cmd = sendCommand (new imapCommand ("AUTHENTICATE", aAuth));
-  KDESasl sasl(aUser, aPass, isSSL ? "imaps" : "imap");
-  sasl.setMethod(aAuth.latin1());
-  while (!cmd->isComplete ())
+  QString firstCommand = aAuth;
+  if ( !challenge.isEmpty() ) { 
+    firstCommand += " ";
+    firstCommand += QString::fromLatin1( challenge.data(), challenge.size() );
+  }
+  cmd = sendCommand (new imapCommand ("AUTHENTICATE", firstCommand.latin1()));
+
+  while ( true )
   {
     //read the next line
     while (parseLoop() == 0);
+    if ( cmd->isComplete() ) break;
 
-    if (!continuation.isEmpty ())
+    if (!continuation.isEmpty())
     {
-      QByteArray challenge;
-      challenge.duplicate(continuation.data() + 2, continuation.size() - 2);
-      challenge.resize(challenge.size() - 2); // trim CRLF
-
-      if (aAuth.upper () == "ANONYMOUS")
-      {
-        // we should present the challenge to the user and ask
-        // him for a mail-address or what ever
-        challenge = KCodecs::base64Encode(aUser.utf8());
-      } else {
-        challenge = sasl.getResponse(challenge);
+//      kdDebug(7116) << "S: " << QCString(continuation.data(),continuation.size()+1) << endl;
+      if ( continuation.size() > 4 ) {
+        tmp.setRawData( continuation.data() + 2, continuation.size() - 4 );
+        KCodecs::base64Decode( tmp, challenge );
+//        kdDebug(7116) << "S-1: " << QCString(challenge.data(),challenge.size()+1) << endl;
+        tmp.resetRawData( continuation.data() + 2, continuation.size() - 4 );
       }
 
+      do {
+        result = sasl_client_step(conn, challenge.isEmpty() ? 0 : challenge.data(),
+                                  challenge.size(),
+                                  &client_interact,
+                                  &out, &outlen);
+
+        if (result == SASL_INTERACT)
+          if ( !sasl_interact( slave, ai, client_interact ) ) {
+            sasl_dispose( &conn );
+            return false;
+          }
+      } while ( result == SASL_INTERACT );
+
+      if ( result != SASL_CONTINUE && result != SASL_OK ) {
+        kdDebug(7116) << "sasl_client_step failed with: " << result << endl;
+        resultInfo = QString::fromUtf8( sasl_errdetail( conn ) );
+        sasl_dispose( &conn );
+        return false;
+      }
+
+      tmp.setRawData( out, outlen );
+//      kdDebug(7116) << "C-1: " << QCString(tmp.data(),tmp.size()+1) << endl;
+      KCodecs::base64Encode( tmp, challenge );
+      tmp.resetRawData( out, outlen );
+//      kdDebug(7116) << "C: " << QCString(challenge.data(),challenge.size()+1) << endl;
       parseWriteLine (challenge);
       continuation.resize(0);
     }
@@ -185,6 +310,8 @@ imapParser::clientAuthenticate (const QString & aUser, const QString & aPass,
   resultInfo = cmd->resultInfo();
   completeQueue.removeRef (cmd);
 
+  sasl_dispose( &conn ); //we don't use sasl_en/decode(), so it's safe to dispose the connection.
+#endif //HAVE_LIBSASL2
   return retVal;
 }
 
@@ -221,6 +348,10 @@ imapParser::parseUntagged (parseString & result)
     {
       parseResult (what, result);
     }
+    else if (qstrncmp(what, "NAMESPACE", what.size()) == 0)
+    {
+      parseNamespace (result);
+    }
     break;
 
   case 'O':                    // OK
@@ -285,11 +416,15 @@ imapParser::parseUntagged (parseString & result)
     }
     break;
 
-  case 'A': // ACL
+  case 'A': // ACL or ANNOTATION
     if (qstrncmp(what, "ACL", what.size()) == 0)
     {
       parseAcl (result);
     }
+    else if (qstrncmp(what, "ANNOTATION", what.size()) == 0)
+    {
+      parseAnnotation (result);
+    }
     break;
   default:
     //better be a number
@@ -318,8 +453,10 @@ imapParser::parseUntagged (parseString & result)
           if (qstrncmp(what, "FETCH", what.size()) == 0)
           {
             seenUid = QString::null;
-            if (lastHandled) lastHandled->clear();
-            else lastHandled = new imapCache();
+            if (lastHandled) 
+              lastHandled->clear();
+            else 
+              lastHandled = new imapCache();
             parseFetch (number, result);
           }
           break;
@@ -352,7 +489,8 @@ void
 imapParser::parseResult (QByteArray & result, parseString & rest,
   const QString & command)
 {
-  if (command == "SELECT") selectInfo.setReadWrite(true);
+  if (command == "SELECT") 
+    selectInfo.setReadWrite(true);
 
   if (rest[0] == '[')
   {
@@ -494,7 +632,8 @@ imapParser::parseResult (QByteArray & result, parseString & rest,
 
 void imapParser::parseCapability (parseString & result)
 {
-  imapCapabilities = QStringList::split (' ', result.cstr().lower());
+  QCString temp( result.cstr() );
+  imapCapabilities = QStringList::split ( ' ', KPIM::kAsciiToLower( temp.data() ) );
 }
 
 void imapParser::parseFlags (parseString & result)
@@ -550,6 +689,24 @@ void imapParser::parseAcl (parseString & result)
   }
 }
 
+void imapParser::parseAnnotation (parseString & result)
+{
+  parseOneWordC (result); // skip mailbox name
+  skipWS (result);
+  parseOneWordC (result); // skip entry name (we know it since we don't allow wildcards in it)
+  skipWS (result);
+  if (result.isEmpty() || result[0] != '(')
+    return;
+  result.pos++;
+  skipWS (result);
+  int outlen = 1;
+  // The result is name1 value1 name2 value2 etc. The caller will sort it out.
+  while ( outlen && !result.isEmpty() && result[0] != ')' ) {
+    QCString word = parseLiteralC (result, false, false, &outlen);
+    lastResults.append (word);
+  }
+}
+
 void imapParser::parseMyRights (parseString & result)
 {
   parseOneWordC (result); // skip mailbox name
@@ -645,9 +802,6 @@ void imapParser::parseAddressList (parseString & inWords, QPtrList<mailAddress>&
 
 const mailAddress& imapParser::parseAddress (parseString & inWords, mailAddress& retVal)
 {
-  // This is already done in parseAddressList
-  //if (inWords[0] != '(')
-  //  return retVal;
   inWords.pos++;
   skipWS (inWords);
 
@@ -665,7 +819,7 @@ const mailAddress& imapParser::parseAddress (parseString & inWords, mailAddress&
 
 mailHeader * imapParser::parseEnvelope (parseString & inWords)
 {
-  mailHeader *envelope = NULL;
+  mailHeader *envelope = 0;
 
   if (inWords[0] != '(')
     return envelope;
@@ -678,7 +832,7 @@ mailHeader * imapParser::parseEnvelope (parseString & inWords)
   envelope->setDate(parseLiteralC(inWords));
 
   //subject
-  envelope->setSubjectEncoded(parseLiteralC(inWords));
+  envelope->setSubject(parseLiteralC(inWords));
 
   QPtrList<mailAddress> list;
   list.setAutoDelete(true);
@@ -818,7 +972,7 @@ mimeHeader * imapParser::parseSimplePart (parseString & inWords,
   parameters.setAutoDelete (true);
 
   if (inWords[0] != '(')
-    return NULL;
+    return 0;
 
   if (!localPart)
     localPart = new mimeHeader;
@@ -1086,7 +1240,7 @@ void imapParser::parseBody (parseString & inWords)
     // parse the header
     if (qstrncmp(specifier, "0", specifier.size()) == 0)
     {
-      mailHeader *envelope = NULL;
+      mailHeader *envelope = 0;
       if (lastHandled)
         envelope = lastHandled->getHeader ();
 
@@ -1115,7 +1269,7 @@ void imapParser::parseBody (parseString & inWords)
       // << QCString(label.data(), label.size()+1) << endl;
       if (qstrncmp(label, "REFERENCES", label.size()) == 0)
       {
-       mailHeader *envelope = NULL;
+       mailHeader *envelope = 0;
        if (lastHandled)
          envelope = lastHandled->getHeader ();
 
@@ -1146,7 +1300,7 @@ void imapParser::parseBody (parseString & inWords)
       if (spec.find(".MIME") != -1)
       {
         mailHeader *envelope = new mailHeader;
-        QString theHeader = parseLiteralC(inWords, true);
+        QString theHeader = parseLiteralC(inWords, false);
         mimeIOQString myIO;
         myIO.setString (theHeader);
         envelope->parseHeader (myIO);
@@ -1162,7 +1316,7 @@ void imapParser::parseBody (parseString & inWords)
   }
   else // no part specifier
   {
-    mailHeader *envelope = NULL;
+    mailHeader *envelope = 0;
     if (lastHandled)
       envelope = lastHandled->getHeader ();
 
@@ -1192,7 +1346,7 @@ void imapParser::parseFetch (ulong /* value */, parseString & inWords)
   skipWS (inWords);
 
   delete lastHandled;
-  lastHandled = NULL;
+  lastHandled = 0;
 
   while (!inWords.isEmpty () && inWords[0] != ')')
   {
@@ -1207,11 +1361,12 @@ void imapParser::parseFetch (ulong /* value */, parseString & inWords)
       case 'E':
         if (word == "ENVELOPE")
         {
-          mailHeader *envelope = NULL;
+          mailHeader *envelope = 0;
 
           if (lastHandled)
             envelope = lastHandled->getHeader ();
-          else lastHandled = new imapCache();
+          else 
+            lastHandled = new imapCache();
 
           if (envelope && !envelope->getMessageId ().isEmpty ())
           {
@@ -1244,7 +1399,7 @@ void imapParser::parseFetch (ulong /* value */, parseString & inWords)
         }
         else if (word == "BODYSTRUCTURE")
         {
-          mailHeader *envelope = NULL;
+          mailHeader *envelope = 0;
 
           if (lastHandled)
             envelope = lastHandled->getHeader ();
@@ -1266,10 +1421,11 @@ void imapParser::parseFetch (ulong /* value */, parseString & inWords)
         if (word == "UID")
         {
           seenUid = parseOneWordC(inWords);
-          mailHeader *envelope = NULL;
+          mailHeader *envelope = 0;
           if (lastHandled)
             envelope = lastHandled->getHeader ();
-          else lastHandled = new imapCache();
+          else 
+            lastHandled = new imapCache();
 
           if (envelope || seenUid.isEmpty ())
           {
@@ -1390,6 +1546,67 @@ void imapParser::parseRecent (ulong value, parseString & result)
   result.pos = result.data.size();
 }
 
+void imapParser::parseNamespace (parseString & result)
+{
+  if ( result[0] != '(' )
+    return;
+
+  QString delimEmpty;
+  if ( imapNamespaceDelimiter.contains( QString::null ) )
+    delimEmpty = imapNamespaceDelimiter[QString::null];
+
+  imapNamespaceDelimiter.clear();
+  imapNamespaces.clear();
+
+  // remember what section we're in (user, other users, shared)
+  int ns = -1;
+  while ( !result.isEmpty() )
+  {
+    if ( result[0] == '(' )
+    {
+      result.pos++; // tie off (
+      if ( result[0] == '(' )
+      {
+        // new namespace section
+        result.pos++; // tie off (
+        ++ns;
+      }
+      // namespace prefix
+      QCString prefix = parseOneWordC( result );
+      // delimiter
+      QCString delim = parseOneWordC( result );
+      kdDebug(7116) << "imapParser::parseNamespace ns='" << prefix <<
+       "',delim='" << delim << "'" << endl;
+      const QString prefixStr = prefix;
+      QString nsentry = QString::number( ns ) + "=" + prefixStr;
+      imapNamespaces.append( nsentry );
+      if ( prefix.right( 1 ) == delim ) {
+        // strip delimiter to get a correct entry for comparisons
+        prefix.resize( prefix.length() );
+      }      
+      imapNamespaceDelimiter[prefix] = delim;
+      
+      result.pos++; // tie off )
+      skipWS( result );
+    } else if ( result[0] == ')' )
+    {
+      result.pos++; // tie off )
+      skipWS( result );
+    } else if ( result[0] == 'N' )
+    {
+      // drop NIL
+      ++ns;
+      parseOneWord( result );
+    } else {
+      // drop whatever it is
+      parseOneWord( result );
+    }
+  }
+  if ( !delimEmpty.isEmpty() ) {
+    imapNamespaceDelimiter[QString::null] = delimEmpty;
+  }
+}
+
 int imapParser::parseLoop ()
 {
   parseString result;
@@ -1507,6 +1724,7 @@ imapParser::parseURL (const KURL & _url, QString & _box, QString & _section,
     parameters = QStringList::split (';', paramString);  //split parameters
     _box.truncate( paramStart ); // strip parameters
   }
+  // extract parameters
   for (QStringList::ConstIterator it (parameters.begin ());
        it != parameters.end (); ++it)
   {
@@ -1516,8 +1734,6 @@ imapParser::parseURL (const KURL & _url, QString & _box, QString & _section,
     int pt = temp.find ('/');
     if (pt > 0)
       temp.truncate(pt);
-//    if(temp[temp.length()-1] == '/')
-//      temp = temp.left(temp.length()-1);
     if (temp.find ("section=", 0, false) == 0)
       _section = temp.right (temp.length () - 8);
     else if (temp.find ("type=", 0, false) == 0)
@@ -1536,6 +1752,7 @@ imapParser::parseURL (const KURL & _url, QString & _box, QString & _section,
 
   if (!_box.isEmpty ())
   {
+    // strip /
     if (_box[0] == '/')
       _box = _box.right (_box.length () - 1);
     if (!_box.isEmpty () && _box[_box.length () - 1] == '/')
@@ -1708,7 +1925,7 @@ bool imapParser::hasCapability (const QString & cap)
        it != imapCapabilities.end (); ++it)
   {
 //    kdDebug(7116) << "imapParser::hasCapability - Examining '" << (*it) << "'" << endl;
-    if (c == *it)
+    if ( !(kasciistricmp(c.ascii(), (*it).ascii())) )
     {
       return true;
     }
@@ -1716,3 +1933,24 @@ bool imapParser::hasCapability (const QString & cap)
   return false;
 }
 
+void imapParser::removeCapability (const QString & cap)
+{
+  imapCapabilities.remove(cap.lower());
+}
+
+QString imapParser::namespaceForBox( const QString & box )
+{
+  kdDebug(7116) << "imapParse::namespaceForBox " << box << endl;
+  QString myNamespace;
+  if ( !box.isEmpty() )
+  {
+    QValueList<QString> list = imapNamespaceDelimiter.keys();
+    for ( QValueList<QString>::Iterator it = list.begin(); it != list.end(); ++it )
+    {
+      if ( !(*it).isEmpty() && box.find( *it ) != -1 )
+        return (*it);
+    }
+  }
+  return myNamespace;
+}
+
