@@ -29,14 +29,13 @@ namespace KSync {
 
 class CalendarSyncee;
 class AddressBookSyncee;
-class EventSyncee;
-class TodoSyncee;
 class BookmarkSyncee;
+class UnknownSyncee;
 
 /**
   This class provides a list of Syncees.
 */
-class SynceeList : public QValueList<Syncee *>
+class KDE_EXPORT SynceeList : public QValueList<Syncee *>
 {
   public:
     SynceeList();
@@ -44,9 +43,13 @@ class SynceeList : public QValueList<Syncee *>
 
     CalendarSyncee *calendarSyncee() const;
     AddressBookSyncee *addressBookSyncee() const;
-    EventSyncee *eventSyncee() const;
-    TodoSyncee *todoSyncee() const;
     BookmarkSyncee *bookmarkSyncee() const;
+    UnknownSyncee  *unknownSyncee()const;
+
+    template<class T>
+    T              *templateSyncee()const;
+
+    void deleteAndClear();
 };
 
 }
