@@ -1,7 +1,7 @@
 /*  -*- mode: C++; c-file-style: "gnu" -*-
  *
  *  This file is part of KMail, the KDE mail client.
- *  Copyright (c) 2002-2003  Bo Thorsen <bo@sonofthor.dk>
+ *  Copyright (c) 2002-2004  Bo Thorsen <bo@sonofthor.dk>
  *                2002-2003  Steffen Hansen <hansen@kde.org>
  *                2002-2003  Zack Rusin <zack@kde.org>
  *
@@ -35,11 +35,15 @@
 #endif
 
 #include "cachedimapjob.h"
+#include "imapaccountbase.h"
 
 #include "kmfoldermgr.h"
+#include "kmfolder.h"
 #include "kmfoldercachedimap.h"
+#include "kmailicalifaceimpl.h"
 #include "kmacctcachedimap.h"
 #include "kmmsgdict.h"
+#include "maildirjob.h"
 
 #include <kio/scheduler.h>
 #include <kio/job.h>
@@ -55,7 +59,7 @@ namespace KMail {
 CachedImapJob::CachedImapJob( const QValueList<MsgForDownload>& msgs,
                               JobType type, KMFolderCachedImap* folder )
   : FolderJob( type ), mFolder( folder ), mMsgsForDownload( msgs ),
-    mTotalBytes(0), mMsg(0)
+    mTotalBytes(0), mMsg(0), mParentFolder( 0 )
 {
   QValueList<MsgForDownload>::ConstIterator it = msgs.begin();
   for ( ; it != msgs.end() ; ++it )
@@ -65,58 +69,66 @@ CachedImapJob::CachedImapJob( const QValueList<MsgForDownload>& msgs,
 // Put messages
 CachedImapJob::CachedImapJob( const QPtrList<KMMessage>& msgs, JobType type,
                               KMFolderCachedImap* folder )
-  : FolderJob( msgs, QString::null, type, folder ), mFolder( folder ),
-    mTotalBytes( 0 ), mMsg( 0 )
+  : FolderJob( msgs, QString::null, type, folder?folder->folder():0 ), mFolder( folder ),
+    mTotalBytes( msgs.count() ), // we abuse it as "total number of messages"
+    mMsg( 0 ), mParentFolder( 0 )
 {
 }
 
 CachedImapJob::CachedImapJob( const QValueList<unsigned long>& msgs,
 			      JobType type, KMFolderCachedImap* folder )
-  : FolderJob( QPtrList<KMMessage>(), QString::null, type, folder ),
-    mFolder( folder ), mSerNumMsgList( msgs ), mTotalBytes( 0 ), mMsg( 0 )
+  : FolderJob( QPtrList<KMMessage>(), QString::null, type, folder?folder->folder():0 ),
+    mFolder( folder ), mSerNumMsgList( msgs ), mTotalBytes( msgs.count() ), mMsg( 0 ),
+    mParentFolder ( 0 )
 {
 }
 
+// Add sub folders
 CachedImapJob::CachedImapJob( const QValueList<KMFolderCachedImap*>& fList,
                               JobType type, KMFolderCachedImap* folder )
-  : FolderJob( type ), mFolder( folder ), mFolderList( fList ), mMsg( 0 )
+  : FolderJob( type ), mFolder( folder ), mFolderList( fList ), mMsg( 0 ),
+    mParentFolder ( 0 )
 {
 }
 
-CachedImapJob::CachedImapJob( const QString& uids, JobType type,
+// Rename folder
+CachedImapJob::CachedImapJob( const QString& string1, JobType type,
                               KMFolderCachedImap* folder )
-  : FolderJob( type ), mFolder(folder), mMsg( 0 ), mString( uids )
+  : FolderJob( type ), mFolder(folder), mMsg( 0 ), mString( string1 ),
+    mParentFolder ( 0 )
 {
   assert( folder );
+  assert( type != tDeleteMessage ); // moved to another ctor
 }
 
-CachedImapJob::CachedImapJob( const QStringList& folderpaths, JobType type,
+// Delete folders or messages
+CachedImapJob::CachedImapJob( const QStringList& foldersOrMsgs, JobType type,
                               KMFolderCachedImap* folder )
-  : FolderJob( type ), mFolder( folder ), mFolderPathList( folderpaths ),
-    mMsg( 0 )
+  : FolderJob( type ), mFolder( folder ), mFoldersOrMessages( foldersOrMsgs ),
+    mMsg( 0 ), mParentFolder( 0 )
 {
   assert( folder );
 }
 
+// Other jobs (list messages,expunge folder, check uid validity)
 CachedImapJob::CachedImapJob( JobType type, KMFolderCachedImap* folder )
-  : FolderJob( type ), mFolder( folder ), mMsg( 0 )
+  : FolderJob( type ), mFolder( folder ), mMsg( 0 ), mParentFolder ( 0 )
 {
   assert( folder );
 }
 
 CachedImapJob::~CachedImapJob()
 {
-  mAccount->displayProgress();
   mAccount->mJobList.remove(this);
 }
 
-void CachedImapJob::init()
+void CachedImapJob::execute()
 {
   mSentBytes = 0;
 
   if( !mFolder ) {
     if( !mMsgList.isEmpty() ) {
-      mFolder = static_cast<KMFolderCachedImap*>(mMsgList.first()->parent());
+      mFolder = static_cast<KMFolderCachedImap*>(mMsgList.first()->storage());
     }
   }
   assert( mFolder );
@@ -137,29 +149,70 @@ void CachedImapJob::init()
   switch( mType ) {
   case tGetMessage:       slotGetNextMessage();     break;
   case tPutMessage:       slotPutNextMessage();     break;
-  case tDeleteMessage:    deleteMessages(mString);  break;
+  case tDeleteMessage:    slotDeleteNextMessages(); break;
   case tExpungeFolder:    expungeFolder();          break;
   case tAddSubfolders:    slotAddNextSubfolder();   break;
   case tDeleteFolders:    slotDeleteNextFolder();   break;
   case tCheckUidValidity: checkUidValidity();       break;
   case tRenameFolder:     renameFolder(mString);    break;
+  case tListMessages:     listMessages();           break;
   default:
     assert( 0 );
   }
 }
 
-void CachedImapJob::deleteMessages( const QString& uids )
+void CachedImapJob::listMessages()
 {
+  KURL url = mAccount->getUrl();
+  url.setPath( mFolder->imapPath() + ";UID=1:*;SECTION=FLAGS RFC822.SIZE");
+
+  KIO::SimpleJob *job = KIO::get(url, false, false);
+  KIO::Scheduler::assignJobToSlave( mAccount->slave(), job );
+  ImapAccountBase::jobData jd( url.url(), mFolder->folder() );
+  jd.cancellable = true;
+  mAccount->insertJob( job, jd );
+  connect( job, SIGNAL( result(KIO::Job *) ),
+           this, SLOT( slotListMessagesResult( KIO::Job* ) ) );
+  // send the data directly for KMFolderCachedImap
+  connect( job, SIGNAL( data( KIO::Job*, const QByteArray& ) ),
+           mFolder, SLOT( slotGetMessagesData( KIO::Job* , const QByteArray& ) ) );
+}
+
+void CachedImapJob::slotDeleteNextMessages( KIO::Job* job )
+{
+  if (job) {
+    KMAcctCachedImap::JobIterator it = mAccount->findJob(job);
+    if ( it == mAccount->jobsEnd() ) { // Shouldn't happen
+      delete this;
+      return;
+    }
+
+    if( job->error() ) {
+      mAccount->handleJobError( job, i18n( "Error while deleting messages on the server: " ) + '\n' );
+      delete this;
+      return;
+    }
+    mAccount->removeJob(it);
+  }
+
+  if( mFoldersOrMessages.isEmpty() ) {
+    // No more messages to delete
+    delete this;
+    return;
+  }
+
+  QString uids = mFoldersOrMessages.front(); mFoldersOrMessages.pop_front();
+
   KURL url = mAccount->getUrl();
   url.setPath( mFolder->imapPath() +
                QString::fromLatin1(";UID=%1").arg(uids) );
 
-  KIO::SimpleJob *job = KIO::file_delete( url, false );
-  KIO::Scheduler::assignJobToSlave( mAccount->slave(), job );
-  ImapAccountBase::jobData jd( url.url(), mFolder );
-  mAccount->insertJob( job, jd );
-  connect( job, SIGNAL( result(KIO::Job *) ),
-           this, SLOT( slotDeleteResult(KIO::Job *) ) );
+  KIO::SimpleJob *simpleJob = KIO::file_delete( url, false );
+  KIO::Scheduler::assignJobToSlave( mAccount->slave(), simpleJob );
+  ImapAccountBase::jobData jd( url.url(), mFolder->folder() );
+  mAccount->insertJob( simpleJob, jd );
+  connect( simpleJob, SIGNAL( result(KIO::Job *) ),
+           this, SLOT( slotDeleteNextMessages(KIO::Job *) ) );
 }
 
 void CachedImapJob::expungeFolder()
@@ -170,24 +223,26 @@ void CachedImapJob::expungeFolder()
 
   KIO::SimpleJob *job = KIO::file_delete( url, false );
   KIO::Scheduler::assignJobToSlave( mAccount->slave(), job );
-  ImapAccountBase::jobData jd( url.url(), mFolder );
+  ImapAccountBase::jobData jd( url.url(), mFolder->folder() );
   mAccount->insertJob( job, jd );
   connect( job, SIGNAL( result(KIO::Job *) ),
-           this, SLOT( slotDeleteResult(KIO::Job *) ) );
+           this, SLOT( slotExpungeResult(KIO::Job *) ) );
 }
 
-void CachedImapJob::slotDeleteResult( KIO::Job * job )
+void CachedImapJob::slotExpungeResult( KIO::Job * job )
 {
   KMAcctCachedImap::JobIterator it = mAccount->findJob(job);
   if ( it == mAccount->jobsEnd() ) { // Shouldn't happen
     delete this;
     return;
   }
-  mAccount->removeJob(it);
 
-  if (job->error())
-    mAccount->slotSlaveError( mAccount->slave(), job->error(),
-                              job->errorText() );
+  if (job->error()) {
+    mErrorCode = job->error();
+    mAccount->handleJobError( job, i18n( "Error while deleting messages on the server: " ) + '\n' );
+  }
+  else
+    mAccount->removeJob(it);
 
   delete this;
 }
@@ -202,23 +257,36 @@ void CachedImapJob::slotGetNextMessage(KIO::Job * job)
     }
 
     if (job->error()) {
-      mAccount->removeJob(it);
-      mAccount->slotSlaveError( mAccount->slave(), job->error(),
-                                job->errorText() );
+      mErrorCode = job->error();
+      mAccount->handleJobError( job, i18n( "Error while retrieving message on the server: " ) + '\n' );
       delete this;
       return;
     }
 
     ulong size = 0;
     if ((*it).data.size() > 0) {
-      QString uid = mMsg->headerField("X-UID");
-      size = mMsg->headerField("X-Length").toULong();
-      mMsg->fromByteArray( (*it).data );
-      mMsg->setHeaderField("X-UID",uid);
-      mMsg->setTransferInProgress( false );
+      ulong uid = mMsg->UID();
+      size = mMsg->msgSizeServer();
+
+      // Convert CR/LF to LF.
+      size_t dataSize = (*it).data.size();
+      dataSize = FolderStorage::crlf2lf( (*it).data.data(), dataSize ); // always <=
+      (*it).data.resize( dataSize );
+
       mMsg->setComplete( true );
-      mFolder->addMsgInternal( mMsg, true );
+      mMsg->fromByteArray( (*it).data );
+      mMsg->setUID(uid);
+      mMsg->setMsgSizeServer(size);
+      mMsg->setTransferInProgress( false );
+      int index = 0;
+      mFolder->addMsgInternal( mMsg, true, &index );
+
+      if ( kmkernel->iCalIface().isResourceFolder( mFolder->folder() ) ) {
+        mFolder->setStatus( index, KMMsgStatusRead, false );
+      }
+
       emit messageRetrieved( mMsg );
+      if ( index > 0 ) mFolder->unGetMsg( index );
     } else {
       emit messageRetrieved( 0 );
     }
@@ -237,14 +305,15 @@ void CachedImapJob::slotGetNextMessage(KIO::Job * job)
   MsgForDownload mfd = mMsgsForDownload.front(); mMsgsForDownload.pop_front();
 
   mMsg = new KMMessage;
-  mMsg->setHeaderField("X-UID",QString::number(mfd.uid));
-  mMsg->setHeaderField("X-Length",QString::number(mfd.size));
+  mMsg->setUID(mfd.uid);
+  mMsg->setMsgSizeServer(mfd.size);
   if( mfd.flags > 0 )
-    KMFolderCachedImap::flagsToStatus(mMsg, mfd.flags);
+    KMFolderImap::flagsToStatus(mMsg, mfd.flags);
   KURL url = mAccount->getUrl();
-  url.setPath(mFolder->imapPath() + QString(";UID=%1;SECTION=FLAGS BODY.PEEK[]").arg(mfd.uid));
+  url.setPath(mFolder->imapPath() + QString(";UID=%1;SECTION=BODY.PEEK[]").arg(mfd.uid));
 
-  ImapAccountBase::jobData jd( url.url(), mFolder );
+  ImapAccountBase::jobData jd( url.url(), mFolder->folder() );
+  jd.cancellable = true;
   mMsg->setTransferInProgress(true);
   KIO::SimpleJob *simpleJob = KIO::get(url, false, false);
   KIO::Scheduler::assignJobToSlave(mAccount->slave(), simpleJob);
@@ -281,7 +350,7 @@ void CachedImapJob::slotPutNextMessage()
     int i = 0;
     KMFolder* aFolder = 0;
     kmkernel->msgDict()->getLocation( serNum, &aFolder, &i );
-    if( mFolder != aFolder )
+    if( mFolder->folder() != aFolder )
       // This message was moved or something
       continue;
     mMsg = mFolder->getMsg( i );
@@ -294,16 +363,17 @@ void CachedImapJob::slotPutNextMessage()
   }
 
   KURL url = mAccount->getUrl();
-  QString flags = KMFolderImap::statusToFlags( mMsg->status() ); 
+  QString flags = KMFolderImap::statusToFlags( mMsg->status() );
   url.setPath( mFolder->imapPath() + ";SECTION=" + flags );
 
-  ImapAccountBase::jobData jd( url.url(), mFolder );
+  ImapAccountBase::jobData jd( url.url(), mFolder->folder() );
 
+  mMsg->setUID( 0 ); // for the index
   QCString cstr(mMsg->asString());
   int a = cstr.find("\nX-UID: ");
   int b = cstr.find('\n', a);
   if (a != -1 && b != -1 && cstr.find("\n\n") > a) cstr.remove(a, b-a);
-  mData.resize(cstr.length() + cstr.contains('\n'));
+  QCString mData(cstr.length() + cstr.contains('\n'));
   unsigned int i = 0;
   for( char *ch = cstr.data(); *ch; ch++ ) {
     if ( *ch == '\n' ) {
@@ -313,6 +383,7 @@ void CachedImapJob::slotPutNextMessage()
     mData.at(i) = *ch; i++;
   }
   jd.data = mData;
+  jd.msgList.append( mMsg );
 
   mMsg->setTransferInProgress(true);
   KIO::SimpleJob *simpleJob = KIO::put(url, 0, false, false, false);
@@ -324,9 +395,13 @@ void CachedImapJob::slotPutNextMessage()
            SLOT( slotPutMessageDataReq(KIO::Job *, QByteArray &) ) );
   connect( simpleJob, SIGNAL( data(KIO::Job *, const QByteArray &) ),
            mFolder, SLOT( slotSimpleData(KIO::Job *, const QByteArray &) ) );
+  connect( simpleJob, SIGNAL(infoMessage(KIO::Job *, const QString &)),
+             SLOT(slotPutMessageInfoData(KIO::Job *, const QString &)) );
+
 }
 
 //-----------------------------------------------------------------------------
+// TODO: port to KIO::StoredTransferJob once it's ok to require kdelibs-3.3
 void CachedImapJob::slotPutMessageDataReq(KIO::Job *job, QByteArray &data)
 {
   KMAcctCachedImap::JobIterator it = mAccount->findJob(job);
@@ -345,6 +420,22 @@ void CachedImapJob::slotPutMessageDataReq(KIO::Job *job, QByteArray &data)
     data.resize(0);
 }
 
+//----------------------------------------------------------------------------
+void CachedImapJob::slotPutMessageInfoData(KIO::Job *job, const QString &data)
+{
+  KMFolderCachedImap * imapFolder = static_cast<KMFolderCachedImap*>(mDestFolder->storage());
+  KMAcctCachedImap *account = imapFolder->account();
+  ImapAccountBase::JobIterator it = account->findJob( job );
+  if ( it == account->jobsEnd() ) return;
+
+  if ( data.find("UID") != -1 && mMsg )
+  {
+    int uid = (data.right(data.length()-4)).toInt();
+    kdDebug( 5006 ) << k_funcinfo << "Server told us uid is: " << uid << endl;
+    mMsg->setUID( uid );
+  }
+}
+
 
 //-----------------------------------------------------------------------------
 void CachedImapJob::slotPutMessageResult(KIO::Job *job)
@@ -356,20 +447,42 @@ void CachedImapJob::slotPutMessageResult(KIO::Job *job)
   }
 
   if ( job->error() ) {
-    QStringList errors = job->detailedErrorStrings();
-    QString myError = "<qt><p><b>" + i18n("Error while uploading message")
-      + "</b></p><p>" + i18n("Could not upload the message %1 on the server from folder %2 with URL %3.").arg((*it).items[0]).arg(mFolder->name()).arg((*it).htmlURL())
-      + "</p><p>" + i18n("This could be because you don't have permission to do this. The error message from the server communication is here:") + "</p>";
-    KMessageBox::error( 0, myError + errors[1] + '\n' + errors[2], errors[0] );
-    mAccount->removeJob(it);
-    delete this;
+    bool cont = mAccount->handlePutError( job, *it, mFolder->folder() );
+    if ( !cont ) {
+      delete this;
+    } else {
+      mMsg = 0;
+      slotPutNextMessage();
+    }
     return;
   }
 
   emit messageStored( mMsg );
+
+  // we abuse those fields, the unit is the number of messages, here
+  ++mSentBytes;
+  emit progress( mSentBytes, mTotalBytes );
+
   int i;
   if( ( i = mFolder->find(mMsg) ) != -1 ) {
-    mFolder->removeMsg(i);
+     /*
+      * If we have aquired a uid during upload the server supports the uidnext
+      * extension and there is no need to redownload this mail, we already have
+      * it. Otherwise remove it, it will be redownloaded.
+      */
+     if ( mMsg->UID() == 0 ) {
+        mFolder->removeMsg(i);
+     } else {
+        // When removing+readding, no point in telling the imap resources about it
+        bool b = kmkernel->iCalIface().isResourceQuiet();
+        kmkernel->iCalIface().setResourceQuiet( true );
+
+        mFolder->take( i );
+        mFolder->addMsgKeepUID( mMsg );
+        mMsg->setTransferInProgress( false );
+
+        kmkernel->iCalIface().setResourceQuiet( b );
+     }
   }
   mMsg = NULL;
   mAccount->removeJob( it );
@@ -386,22 +499,22 @@ void CachedImapJob::slotAddNextSubfolder( KIO::Job * job )
       return;
     }
 
-    if ( job->error() &&
-         !static_cast<KMFolderCachedImap*>((*it).parent)->silentUpload() ) {
-      QStringList errors = job->detailedErrorStrings();
-      QString myError = "<qt><p><b>" + i18n("Error while uploading folder")
-        + "</b></p><p>" + i18n("Could not make the folder %1 on the server.").arg((*it).items[0])
-        + "</p><p>" + i18n("This could be because you don't have permission to do this or because the folder is already present on the server. The error message from the server communication is here:") + "</p>";
-      KMessageBox::error( 0, myError + errors[1] + '\n' + errors[2],
-                          errors[0] );
+    // make copy of setting, to reset it before potentially destroying 'it'
+    bool silentUpload = static_cast<KMFolderCachedImap*>((*it).parent->storage())->silentUpload();
+    static_cast<KMFolderCachedImap*>((*it).parent->storage())->setSilentUpload( false );
+
+    if ( job->error() && !silentUpload ) {
+      QString myError = "<p><b>" + i18n("Error while uploading folder")
+        + "</b></p><p>" + i18n("Could not make the folder <b>%1</b> on the server.").arg((*it).items[0])
+        + "</p><p>" + i18n("This could be because you do not have permission to do this, or because the folder is already present on the server; the error message from the server communication is here:") + "</p>";
+      mAccount->handleJobError( job, myError );
     }
-    static_cast<KMFolderCachedImap*>((*it).parent)->setSilentUpload( false );
-    mAccount->removeJob( it );
 
     if( job->error() ) {
       delete this;
       return;
     }
+    mAccount->removeJob( it );
   }
 
   if (mFolderList.isEmpty()) {
@@ -413,9 +526,13 @@ void CachedImapJob::slotAddNextSubfolder( KIO::Job * job )
   KMFolderCachedImap *folder = mFolderList.front();
   mFolderList.pop_front();
   KURL url = mAccount->getUrl();
-  url.setPath(mFolder->imapPath() + folder->name());
+  url.setPath(mFolder->imapPath() + folder->folder()->name());
 
-  ImapAccountBase::jobData jd( url.url(), folder );
+  // Associate the jobData with the parent folder, not with the child
+  // This is necessary in case of an error while creating the subfolder,
+  // so that folderComplete is called on the parent (and the sync resetted).
+  ImapAccountBase::jobData jd( url.url(), mFolder->folder() );
+  jd.items << folder->label(); // for the err msg
   KIO::SimpleJob *simpleJob = KIO::mkdir(url);
   KIO::Scheduler::assignJobToSlave(mAccount->slave(), simpleJob);
   mAccount->insertJob(simpleJob, jd);
@@ -432,25 +549,28 @@ void CachedImapJob::slotDeleteNextFolder( KIO::Job *job )
       delete this;
       return;
     }
-    mAccount->removeJob(it);
+
+    mAccount->removeDeletedFolder( (*it).path );
 
     if( job->error() ) {
-      job->showErrorDialog( 0L  ); // why not mAccount->slotSlaveError?
+      mAccount->handleJobError( job, i18n( "Error while deleting folder %1 on the server: " ).arg( (*it).path ) + '\n' );
       delete this;
       return;
     }
+    mAccount->removeJob(it);
   }
 
-  if( mFolderPathList.isEmpty() ) {
+  if( mFoldersOrMessages.isEmpty() ) {
     // No more folders to delete
     delete this;
     return;
   }
 
-  QString folderPath = mFolderPathList.front(); mFolderPathList.pop_front();
+  QString folderPath = mFoldersOrMessages.front(); mFoldersOrMessages.pop_front();
   KURL url = mAccount->getUrl();
   url.setPath(folderPath);
-  ImapAccountBase::jobData jd( url.url(), mFolder );
+  ImapAccountBase::jobData jd( url.url(), mFolder->folder() );
+  jd.path = url.path();
   KIO::SimpleJob *simpleJob = KIO::file_delete(url, false);
   KIO::Scheduler::assignJobToSlave(mAccount->slave(), simpleJob);
   mAccount->insertJob(simpleJob, jd);
@@ -463,7 +583,8 @@ void CachedImapJob::checkUidValidity()
   KURL url = mAccount->getUrl();
   url.setPath( mFolder->imapPath() + ";UID=0:0" );
 
-  ImapAccountBase::jobData jd( url.url(), mFolder );
+  ImapAccountBase::jobData jd( url.url(), mFolder->folder() );
+  jd.cancellable = true;
 
   KIO::SimpleJob *job = KIO::get( url, false, false );
   KIO::Scheduler::assignJobToSlave( mAccount->slave(), job );
@@ -483,8 +604,8 @@ void CachedImapJob::slotCheckUidValidityResult(KIO::Job * job)
   }
 
   if( job->error() ) {
-    mAccount->removeJob(it);
-    job->showErrorDialog( 0 ); // why not mAccount->slotSlaveError?
+    mErrorCode = job->error();
+    mAccount->handleJobError( job, i18n( "Error while reading folder %1 on the server: " ).arg( (*it).parent->label() ) + '\n' );
     delete this;
     return;
   }
@@ -497,22 +618,24 @@ void CachedImapJob::slotCheckUidValidityResult(KIO::Job * job)
     // TODO: Tell the user that he has a problem
     kdDebug(5006) << "No uidvalidity available for folder "
                   << mFolder->name() << endl;
-    return;
   }
-  int b = cstr.find("\r\n", a);
-  if ( (b - a - 15) >= 0 ) {
-    QString uidv = cstr.mid(a + 15, b - a - 15);
-    // kdDebug(5006) << "New uidv = " << uidv << ", old uidv = "
-    //               << mFolder->uidValidity() << endl;
-    if( !mFolder->uidValidity().isEmpty() && mFolder->uidValidity() != uidv ) {
-      // kdDebug(5006) << "Expunging the mailbox " << mFolder->name()
-      //               << "!" << endl;
-      mFolder->expunge();
-      mFolder->setLastUid( 0 );
-    }
-  } else
-    kdDebug(5006) << "No uidvalidity available for folder "
-                  << mFolder->name() << endl;
+  else {
+    int b = cstr.find("\r\n", a);
+    if ( (b - a - 15) >= 0 ) {
+      QString uidv = cstr.mid(a + 15, b - a - 15);
+      // kdDebug(5006) << "New uidv = " << uidv << ", old uidv = "
+      //               << mFolder->uidValidity() << endl;
+      if( !mFolder->uidValidity().isEmpty() && mFolder->uidValidity() != uidv ) {
+        // kdDebug(5006) << "Expunging the mailbox " << mFolder->name()
+        //               << "!" << endl;
+        mFolder->expunge();
+        mFolder->setLastUid( 0 );
+        mFolder->clearUidMap();
+      }
+    } else
+      kdDebug(5006) << "No uidvalidity available for folder "
+                    << mFolder->name() << endl;
+  }
 
   mAccount->removeJob(it);
   delete this;
@@ -529,11 +652,11 @@ void CachedImapJob::renameFolder( const QString &newName )
   KURL urlDst = mAccount->getUrl();
   QString imapPath( mFolder->imapPath() );
   // Destination url = old imappath - oldname + new name
-  imapPath.truncate( imapPath.length() - mFolder->name().length() - 1);
+  imapPath.truncate( imapPath.length() - mFolder->folder()->name().length() - 1);
   imapPath += newName + '/';
   urlDst.setPath( imapPath );
 
-  ImapAccountBase::jobData jd( newName, mFolder );
+  ImapAccountBase::jobData jd( newName, mFolder->folder() );
   jd.path = imapPath;
 
   KIO::SimpleJob *simpleJob = KIO::rename( urlSrc, urlDst, false );
@@ -551,7 +674,7 @@ static void renameChildFolders( KMFolderDir* dir, const QString& oldPath,
     while( node ) {
       if( !node->isDir() ) {
         KMFolderCachedImap* imapFolder =
-          static_cast<KMFolderCachedImap*>(node);
+          static_cast<KMFolderCachedImap*>(static_cast<KMFolder*>(node)->storage());
         if ( !imapFolder->imapPath().isEmpty() )
           // Only rename folders that have been accepted by the server
           if( imapFolder->imapPath().find( oldPath ) == 0 ) {
@@ -559,7 +682,7 @@ static void renameChildFolders( KMFolderDir* dir, const QString& oldPath,
             p = p.mid( oldPath.length() );
             p.prepend( newPath );
             imapFolder->setImapPath( p );
-            renameChildFolders( imapFolder->child(), oldPath, newPath );
+            renameChildFolders( imapFolder->folder()->child(), oldPath, newPath );
           }
       }
       node = dir->next();
@@ -575,35 +698,58 @@ void CachedImapJob::slotRenameFolderResult( KIO::Job *job )
     return;
   }
 
+
   if( job->error() ) {
-    job->showErrorDialog( 0 ); // why not mAccount->slotSlaveError?
+    // Error, revert label change
+    QMap<QString, KMAcctCachedImap::RenamedFolder>::ConstIterator renit = mAccount->renamedFolders().find( mFolder->imapPath() );
+    Q_ASSERT( renit != mAccount->renamedFolders().end() );
+    if ( renit != mAccount->renamedFolders().end() ) {
+      mFolder->folder()->setLabel( (*renit).mOldLabel );
+      mAccount->removeRenamedFolder( mFolder->imapPath() );
+    }
+    mAccount->handleJobError( job, i18n( "Error while trying to rename folder %1" ).arg( mFolder->label() ) + '\n' );
   } else {
-    // Okay, the folder seems to be renamed on the folder,
+    // Okay, the folder seems to be renamed on the server,
     // now rename it on disk
     QString oldName = mFolder->name();
     QString oldPath = mFolder->imapPath();
+    mAccount->removeRenamedFolder( oldPath );
     mFolder->setImapPath( (*it).path );
-    mFolder->KMFolder::rename( (*it).url );
+    mFolder->FolderStorage::rename( (*it).url );
 
     if( oldPath.endsWith( "/" ) ) oldPath.truncate( oldPath.length() -1 );
     QString newPath = mFolder->imapPath();
     if( newPath.endsWith( "/" ) ) newPath.truncate( newPath.length() -1 );
-    renameChildFolders( mFolder->child(), oldPath, newPath );
+    renameChildFolders( mFolder->folder()->child(), oldPath, newPath );
     kmkernel->dimapFolderMgr()->contentsChanged();
-  }
 
-  mAccount->removeJob(it);
+    mAccount->removeJob(it);
+  }
   delete this;
 }
 
-void CachedImapJob::execute()
+void CachedImapJob::slotListMessagesResult( KIO::Job * job )
 {
-  init();
+  KMAcctCachedImap::JobIterator it = mAccount->findJob(job);
+  if ( it == mAccount->jobsEnd() ) { // Shouldn't happen
+    delete this;
+    return;
+  }
+
+  if (job->error()) {
+    mErrorCode = job->error();
+    mAccount->handleJobError( job, i18n( "Error while deleting messages on the server: " ) + '\n' );
+  }
+  else
+    mAccount->removeJob(it);
+
+  delete this;
 }
 
-void CachedImapJob::expireMessages()
+//-----------------------------------------------------------------------------
+void CachedImapJob::setParentFolder( const KMFolderCachedImap* parent )
 {
-  //FIXME: not implemented yet
+  mParentFolder = const_cast<KMFolderCachedImap*>( parent );
 }
 
 }
