@@ -2,6 +2,8 @@
 
 #include <config.h>
 
+#include "kmsender.h"
+
 #include <kmime_header_parsing.h>
 using namespace KMime::Types;
 
@@ -23,22 +25,24 @@ using namespace KMime::Types;
 #include "kmfiltermgr.h"
 
 #include "kcursorsaver.h"
-#include "kmsender.h"
-#include "kmidentity.h"
-#include "identitymanager.h"
-#include "kmbroadcaststatus.h"
+#include <libkpimidentities/identity.h>
+#include <libkpimidentities/identitymanager.h>
+#include "progressmanager.h"
 #include "kmaccount.h"
 #include "kmtransport.h"
 #include "kmfolderindex.h"
 #include "kmfoldermgr.h"
 #include "kmmsgdict.h"
 #include "kmmsgpart.h"
+#include "protocols.h"
+#include "kmcommands.h"
 #include <mimelib/mediatyp.h>
 
 #define SENDER_GROUP "sending mail"
 
 //-----------------------------------------------------------------------------
 KMSender::KMSender()
+  : mOutboxFolder( 0 ), mSentFolder( 0 )
 {
   mPrecommand = 0;
   mSendProc = 0;
@@ -53,6 +57,7 @@ KMSender::KMSender()
   mFailedMessages = 0;
   mSentBytes = 0;
   mTotalBytes = 0;
+  mProgressItem = 0;
 }
 
 
@@ -68,7 +73,8 @@ KMSender::~KMSender()
 //-----------------------------------------------------------------------------
 void KMSender::setStatusMsg(const QString &msg)
 {
-  KMBroadcastStatus::instance()->setStatusMsg(msg);
+  if ( mProgressItem )
+    mProgressItem->setStatus(msg);
 }
 
 //-----------------------------------------------------------------------------
@@ -123,28 +129,41 @@ bool KMSender::send(KMMessage* aMsg, short sendNow)
     // RFC822 says:
     // Note that the "Bcc" field may be empty, while the "To" field is required to
     // have at least one address.
-    return FALSE;
+    //
+    // however:
+    //
+    // The following string is accepted according to RFC 2822,
+    // section 3.4 "Address Specification" where they say:
+    //
+    //     "An address may either be an individual mailbox,
+    //      or a group of mailboxes."
+    // and:
+    //     "group   +   display-name ":" [mailbox-list / CFWS] ";"
+    //      [CFWS]"
+    //
+    // In this syntax our "undisclosed-recipients: ;"
+    // just specifies an empty group.
+    //
+    // In further explanations RFC 2822 states that it *is*
+    // allowed to have a ZERO number of mailboxes in the "mailbox-list".
+    aMsg->setTo("Undisclosed.Recipients: ;");
   }
 
-  QString msgId = KMMessage::generateMessageId( aMsg->sender() );
+
+  // Handle redirections
+  QString from  = aMsg->headerField("X-KMail-Redirect-From");
+  QString msgId = aMsg->msgId();
+  if( from.isEmpty() || msgId.isEmpty() ) {
+    msgId = KMMessage::generateMessageId( aMsg->sender() );
     //kdDebug(5006) << "Setting Message-Id to '" << msgId << "'\n";
     aMsg->setMsgId( msgId );
+  }
 
   if (sendNow==-1) sendNow = mSendImmediate;
 
   kmkernel->outboxFolder()->open();
   aMsg->setStatus(KMMsgStatusQueued);
 
-  // Handle redirections
-  QString f = aMsg->headerField("X-KMail-Redirect-From");
-  if(!f.isEmpty()) {
-    uint id = aMsg->headerField("X-KMail-Identity").stripWhiteSpace().toUInt();
-    const KMIdentity & ident =
-      kmkernel->identityManager()->identityForUoidOrDefault( id );
-    aMsg->setFrom(f + QString(" (by way of %1 <%2>)")
-      .arg(ident.fullName()).arg(ident.emailAddr()));
-  }
-
   rc = kmkernel->outboxFolder()->addMsg(aMsg);
   if (rc)
   {
@@ -185,23 +204,25 @@ bool KMSender::sendQueued(void)
   }
 
   // open necessary folders
-  KMFolder* outbox = kmkernel->outboxFolder();
-  outbox->open();
-  mTotalMessages = outbox->count();
+  mOutboxFolder = kmkernel->outboxFolder();
+  mOutboxFolder->open();
+  mTotalMessages = mOutboxFolder->count();
   if (mTotalMessages == 0) {
     // Nothing in the outbox. We are done.
-    outbox->close();
+    mOutboxFolder->close();
+    mOutboxFolder = 0;
     return TRUE;
   }
   mTotalBytes = 0;
   for( int i = 0 ; i<mTotalMessages ; ++i )
-      mTotalBytes += outbox->getMsgBase(i)->msgSize();
+      mTotalBytes += mOutboxFolder->getMsgBase(i)->msgSize();
 
-  connect(outbox, SIGNAL(msgAdded(int)),
-          this, SLOT(outboxMsgAdded(int)));
+  connect( mOutboxFolder, SIGNAL(msgAdded(int)),
+           this, SLOT(outboxMsgAdded(int)) );
   mCurrentMsg = 0;
 
-  kmkernel->sentFolder()->open();
+  mSentFolder = kmkernel->sentFolder();
+  mSentFolder->open();
   kmkernel->filterMgr()->ref();
 
   // start sending the messages
@@ -214,7 +235,7 @@ void KMSender::emitProgressInfo( int currentFileProgress )
 {
   int percent = (mTotalBytes) ? ( 100 * (mSentBytes+currentFileProgress) / mTotalBytes ) : 0;
   if (percent > 100) percent = 100;
-  KMBroadcastStatus::instance()->setStatusProgressPercent("Sender", percent);
+  mProgressItem->setProgress(percent);
 }
 
 //-----------------------------------------------------------------------------
@@ -230,14 +251,13 @@ void KMSender::doSendMsg()
       mSentMessages++;
       mSentBytes += mCurrentMsg->msgSize();
   }
-  emitProgressInfo( 0 );
 
   // Post-process sent message (filtering)
   if (mCurrentMsg  && kmkernel->filterMgr())
   {
     mCurrentMsg->setTransferInProgress( FALSE );
     if( mCurrentMsg->hasUnencryptedMsg() ) {
-kdDebug(5006) << "KMSender::doSendMsg() post-processing: replace mCurrentMsg body by unencryptedMsg data" << endl;
+      kdDebug(5006) << "KMSender::doSendMsg() post-processing: replace mCurrentMsg body by unencryptedMsg data" << endl;
       // delete all current body parts
       mCurrentMsg->deleteBodyParts();
       // copy Content-[..] headers from unencrypted message to current one
@@ -260,24 +280,26 @@ kdDebug(5006) << "KMSender::doSendMsg() post-processing: replace mCurrentMsg bod
     }
     mCurrentMsg->setStatus(KMMsgStatusSent);
     mCurrentMsg->setStatus(KMMsgStatusRead); // otherwise it defaults to new on imap
+    mCurrentMsg->updateAttachmentState();
 
-    const KMIdentity & id = kmkernel->identityManager()
+    const KPIM::Identity & id = kmkernel->identityManager()
       ->identityForUoidOrDefault( mCurrentMsg->headerField( "X-KMail-Identity" ).stripWhiteSpace().toUInt() );
-    bool folderGone = false;
     if ( !mCurrentMsg->fcc().isEmpty() )
     {
       sentFolder = kmkernel->folderMgr()->findIdString( mCurrentMsg->fcc() );
       if ( sentFolder == 0 )
       // This is *NOT* supposed to be imapSentFolder!
-        sentFolder = 
+        sentFolder =
           kmkernel->dimapFolderMgr()->findIdString( mCurrentMsg->fcc() );
       if ( sentFolder == 0 )
         imapSentFolder =
           kmkernel->imapFolderMgr()->findIdString( mCurrentMsg->fcc() );
-      if ( !sentFolder && !imapSentFolder )
-        folderGone = true;
     }
-    else if ( !id.fcc().isEmpty() )
+    // No, or no usable sentFolder, and no, or no usable imapSentFolder, 
+    // let's try the on in the identity
+    if ( ( sentFolder == 0 || sentFolder->isReadOnly() )
+      && ( imapSentFolder == 0 || imapSentFolder->isReadOnly() ) 
+      && !id.fcc().isEmpty() )
     {
       sentFolder = kmkernel->folderMgr()->findIdString( id.fcc() );
       if ( sentFolder == 0 )
@@ -285,17 +307,12 @@ kdDebug(5006) << "KMSender::doSendMsg() post-processing: replace mCurrentMsg bod
         sentFolder = kmkernel->dimapFolderMgr()->findIdString( id.fcc() );
       if ( sentFolder == 0 )
         imapSentFolder = kmkernel->imapFolderMgr()->findIdString( id.fcc() );
-      if ( !sentFolder && !imapSentFolder )
-        folderGone = true;
     }
-    if (imapSentFolder && imapSentFolder->noContent()) imapSentFolder = 0;
-    if (folderGone)
-      KMessageBox::information(0, i18n("The custom sent-mail folder for identity "
-            "\"%1\" doesn't exist (anymore). "
-            "Therefore the default sent-mail folder "
-            "will be used.").arg( id.identityName() ) );
+    if (imapSentFolder
+        && ( imapSentFolder->noContent() || imapSentFolder->isReadOnly() ) )
+        imapSentFolder = 0;
 
-    if ( sentFolder == 0 ) 
+    if ( sentFolder == 0 || sentFolder->isReadOnly() )
       sentFolder = kmkernel->sentFolder();
 
     if ( sentFolder ) {
@@ -306,13 +323,13 @@ kdDebug(5006) << "KMSender::doSendMsg() post-processing: replace mCurrentMsg bod
       }
     }
 
-    // Disable the emitting of msgAdded signal, because the message is taken out of the 
+    // Disable the emitting of msgAdded signal, because the message is taken out of the
     // current folder (outbox) and re-added, to make filter actions changing the message
     // work. We don't want that to screw up message counts.
     if ( mCurrentMsg->parent() ) mCurrentMsg->parent()->quiet( true );
     int processResult = kmkernel->filterMgr()->process(mCurrentMsg,KMFilterMgr::Outbound);
     if ( mCurrentMsg->parent() ) mCurrentMsg->parent()->quiet( false );
-    
+
     // 0==processed ok, 1==no filter matched, 2==critical error, abort!
     switch (processResult) {
     case 2:
@@ -335,7 +352,12 @@ kdDebug(5006) << "KMSender::doSendMsg() post-processing: replace mCurrentMsg bod
         cleanup();
         return;
       }
-      if (imapSentFolder) imapSentFolder->moveMsg(mCurrentMsg);
+      if (imapSentFolder) {
+        // Does proper folder refcounting and message locking
+        KMCommand *command = new KMMoveCommand( imapSentFolder, mCurrentMsg );
+        command->keepFolderOpen( sentFolder ); // will open it, and close it once done
+        command->start();
+      }
     default:
       break;
     }
@@ -353,7 +375,32 @@ kdDebug(5006) << "KMSender::doSendMsg() post-processing: replace mCurrentMsg bod
   }
 
   // See if there is another queued message
-  mCurrentMsg = kmkernel->outboxFolder()->getMsg(mFailedMessages);
+  mCurrentMsg = mOutboxFolder->getMsg(mFailedMessages);
+  if ( mCurrentMsg && !mCurrentMsg->transferInProgress() &&
+       mCurrentMsg->sender().isEmpty() ) {
+    // if we do not have a sender address then use the email address of the
+    // message's identity or of the default identity unless those two are also
+    // empty
+    const KPIM::Identity & id = kmkernel->identityManager()
+      ->identityForUoidOrDefault( mCurrentMsg->headerField( "X-KMail-Identity" ).stripWhiteSpace().toUInt() );
+    if ( !id.emailAddr().isEmpty() ) {
+      mCurrentMsg->setFrom( id.fullEmailAddr() );
+    }
+    else if ( !kmkernel->identityManager()->defaultIdentity().emailAddr().isEmpty() ) {
+      mCurrentMsg->setFrom( kmkernel->identityManager()->defaultIdentity().fullEmailAddr() );
+    }
+    else {
+      KMessageBox::sorry( 0, i18n( "It's not possible to send messages "
+                                   "without specifying a sender address.\n"
+                                   "Please set the email address of "
+                                   "identity '%1' in the Identities "
+                                   "section of the configuration dialog "
+                                   "and then try again." )
+                             .arg( id.identityName() ) );
+      mOutboxFolder->unGetMsg( mFailedMessages );
+      mCurrentMsg = 0;
+    }
+  }
   if (!mCurrentMsg || mCurrentMsg->transferInProgress())
   {
     // a message is locked finish the send
@@ -380,14 +427,16 @@ kdDebug(5006) << "KMSender::doSendMsg() post-processing: replace mCurrentMsg bod
   // start the sender process or initialize communication
   if (!mSendInProgress)
   {
-    KMBroadcastStatus::instance()->reset();
-    KMBroadcastStatus::instance()->setStatusProgressEnable( "Sender", true );
-    connect(KMBroadcastStatus::instance(), SIGNAL(signalAbortRequested()),
-      SLOT(slotAbortSend()));
+    Q_ASSERT( !mProgressItem );
+    mProgressItem = KPIM::ProgressManager::createProgressItem(
+      "Sender",
+      i18n( "Sending messages" ),
+      i18n("Initiating sender process..."),
+      true );
+    connect( mProgressItem, SIGNAL(progressItemCanceled(KPIM::ProgressItem*)),
+             this, SLOT( slotAbortSend() ) );
     kapp->ref();
-
     mSendInProgress = TRUE;
-    setStatusMsg(i18n("Initiating sender process..."));
   }
 
   QString msgTransport = mCurrentMsg->headerField("X-KMail-Transport");
@@ -405,6 +454,9 @@ kdDebug(5006) << "KMSender::doSendMsg() post-processing: replace mCurrentMsg bod
     mSendProc = createSendProcFromString(msgTransport);
     mMethodStr = msgTransport;
 
+    if( mTransportInfo->encryption == "TLS" || mTransportInfo->encryption == "SSL" )
+      mProgressItem->setUsesCrypto( true );
+
     if (!mSendProc)
       sendProcStarted(false);
     else {
@@ -467,6 +519,11 @@ void KMSender::doSendMsgAux()
 	       .arg(mCurrentMsg->subject()));
   if (!mSendProc->send(mCurrentMsg))
   {
+    if ( mCurrentMsg )
+      mCurrentMsg->setTransferInProgress( false );
+    if ( mOutboxFolder )
+      mOutboxFolder->unGetMsg( mFailedMessages );
+    mCurrentMsg = 0;
     cleanup();
     setStatusMsg(i18n("Failed to send (some) queued messages."));
     return;
@@ -479,6 +536,7 @@ void KMSender::doSendMsgAux()
 //-----------------------------------------------------------------------------
 void KMSender::cleanup(void)
 {
+  kdDebug(5006) << k_funcinfo << endl;
   if (mSendProc && mSendProcStarted) mSendProc->finish(true);
   mSendProc = 0;
   mSendProcStarted = FALSE;
@@ -489,22 +547,30 @@ void KMSender::cleanup(void)
     mCurrentMsg->setTransferInProgress( FALSE );
     mCurrentMsg = 0;
   }
-  disconnect(kmkernel->outboxFolder(), SIGNAL(msgAdded(int)),
-             this, SLOT(outboxMsgAdded(int)));
-  kmkernel->sentFolder()->close();
-  kmkernel->outboxFolder()->close();
-  if (kmkernel->outboxFolder()->count()<0)
-    kmkernel->outboxFolder()->expunge();
-  else kmkernel->outboxFolder()->compact();
+  if ( mSentFolder ) {
+    mSentFolder->close();
+    mSentFolder = 0;
+  }
+  if ( mOutboxFolder ) {
+    disconnect( mOutboxFolder, SIGNAL(msgAdded(int)),
+                this, SLOT(outboxMsgAdded(int)) );
+    mOutboxFolder->close();
+    if ( mOutboxFolder->count( true ) == 0 ) {
+      mOutboxFolder->expunge();
+    }
+    else if ( mOutboxFolder->needsCompacting() ) {
+      mOutboxFolder->compact( KMFolder::CompactSilentlyNow );
+    }
+    mOutboxFolder = 0;
+  }
 
   mSendAborted = false;
   mSentMessages = 0;
   mFailedMessages = 0;
   mSentBytes = 0;
-  disconnect(KMBroadcastStatus::instance(), SIGNAL(signalAbortRequested()),
-    this, SLOT(slotAbortSend()));
-  KMBroadcastStatus::instance()->setStatusProgressEnable( "Sender", false );
-  KMBroadcastStatus::instance()->reset();
+  if ( mProgressItem )
+    mProgressItem->setComplete();
+  mProgressItem = 0;
   kmkernel->filterMgr()->deref();
 }
 
@@ -530,6 +596,12 @@ void KMSender::slotIdle()
 
   if (mSendAborted) {
     // sending of message aborted
+    if ( mCurrentMsg ) {
+      mCurrentMsg->setTransferInProgress( false );
+      if ( mOutboxFolder )
+        mOutboxFolder->unGetMsg( mFailedMessages );
+      mCurrentMsg = 0;
+    }
     msg = i18n("Sending aborted:\n%1\n"
         "The message will stay in the 'outbox' folder until you either "
         "fix the problem (e.g. a broken address) or remove the message "
@@ -541,7 +613,10 @@ void KMSender::slotIdle()
     setStatusMsg( i18n( "Sending aborted." ) );
   } else {
     if (!mSendProc->sendOk()) {
-      mCurrentMsg->setTransferInProgress( false );
+      if ( mCurrentMsg )
+        mCurrentMsg->setTransferInProgress( false );
+      if ( mOutboxFolder )
+        mOutboxFolder->unGetMsg( mFailedMessages );
       mCurrentMsg = 0;
       mFailedMessages++;
       // Sending of message failed.
@@ -558,8 +633,8 @@ void KMSender::slotIdle()
             .arg(errString)
             .arg(mMethodStr);
           res = KMessageBox::warningYesNo( 0 , msg ,
-                  i18n( "Continue sending" ), i18n( "&Continue sending" ),
-                  i18n("&Abort sending") );
+                  i18n( "Continue Sending" ), i18n( "&Continue Sending" ),
+                  i18n("&Abort Sending") );
         } else {
           msg = i18n("Sending failed:\n%1\n"
             "The message will stay in the 'outbox' folder until you either "
@@ -625,7 +700,7 @@ KMSendProc* KMSender::createSendProcFromString(QString transport)
   {
     mTransportInfo->readConfig(nr);
   } else {
-    if (transport.startsWith("smtp://"))
+    if (transport.startsWith("smtp://")) // should probably use KURL and SMTP_PROTOCOL
     {
       mTransportInfo->type = "smtp";
       mTransportInfo->auth = FALSE;
@@ -640,7 +715,7 @@ KMSendProc* KMSender::createSendProcFromString(QString transport)
         mTransportInfo->port = "25";
       }
     } else
-    if (transport.startsWith("smtps://"))
+    if (transport.startsWith("smtps://"))  // should probably use KURL and SMTPS_PROTOCOL
     {
       mTransportInfo->type = "smtps";
       mTransportInfo->auth = FALSE;
@@ -687,14 +762,22 @@ void KMSender::setStatusByLink(const KMMessage *aMsg)
       break;
     n++;
 
-    KMFolder *folder;
-    int index;
+    KMFolder *folder = 0;
+    int index = -1;
     kmkernel->msgDict()->getLocation(msn, &folder, &index);
-
-    if (folder) {
+    if (folder && index != -1) {
       folder->open();
-      folder->setStatus(index, status);
+      if ( status == KMMsgStatusDeleted ) {
+        // Move the message to the trash folder
+        KMDeleteMsgCommand *cmd = 
+          new KMDeleteMsgCommand( folder, folder->getMsg( index ) ); 
+        cmd->start();
+      } else {
+        folder->setStatus(index, status);
+      }
       folder->close();
+    } else {
+      kdWarning(5006) << k_funcinfo << "Cannot update linked message, it could not be found!" << endl;
     }
   }
 }
@@ -736,102 +819,6 @@ bool KMSendProc::finish(bool destructive)
   return TRUE;
 }
 
-#if !KDE_IS_VERSION( 3, 1, 90 ) // dotstuffing and LF->CRLF is not
-				// done by the SMTP kioslave
-QCString KMSendProc::prepareStr(const QCString &aStr, bool toCRLF,
- bool noSingleDot)
-{
-  int tlen;
-  const int len = aStr.length();
-
-  if (aStr.isEmpty()) return QCString();
-
-  QCString target( "" );
-
-  if ( toCRLF ) {
-    // (mmutz) headroom so we actually don't need to resize the target
-    // array. Five percent should suffice. I measured a mean line
-    // length of 42 (no joke) over the my last month's worth of mails.
-    tlen = int(len * 1.05);
-    target.resize( tlen );
-
-    QCString::Iterator t = target.begin();
-    QCString::Iterator te = target.end();
-    te -= 5; // 4 is the max. #(chars) appended in one round, plus one for the \0.
-    QCString::ConstIterator s = aStr.begin();
-    while( (*s) ) {
-
-      char c = *s++;
-
-      if ( c == '\n' ) {
-	*t++ = '\r';
-	*t++ = c;
-
-	if ( noSingleDot && (*s) == '.' ) {
-	  s++;
-	  *t++ = '.';
-	  *t++ = '.';
-	}
-      } else
-	*t++ = c;
-
-      if ( t >= te ) { // nearing the end of the target buffer.
-	int tskip = t - target.begin();
-	tlen += QMAX( len/128, 128 );
-	if ( !target.resize( tlen ) )
-	  // OOM, what else can we do?
-	  return aStr;
-	t = target.begin() + tskip;
-      }
-    }
-    *t = '\0';
-  } else {
-    if ( !noSingleDot ) return aStr;
-
-    tlen = 0;
-
-    QCString::Iterator t = target.begin();
-    QCString::ConstIterator olds = aStr.begin();
-    QCString::ConstIterator s = aStr.begin();
-
-    while ( (*s) ) {
-      if ( *s++ == '\n' && *s == '.' ) {
-
-	int skip = s - olds + 1;
-
-	if ( tlen ) {
-	  if ( tlen + skip >= (int)target.size() ) {
-	    // resize to 128 + <currently used> + <yet to be copied>
-	    target.resize( 128 + tlen + len - ( olds - aStr.begin() ) );
-	    t = target.begin() + tlen;
-	  }
-	} else {
-	  target.resize( int( len * 1.02 ) );
-	  t = target.begin();
-	}
-
-	memcpy( t, olds, skip );
-	tlen += skip; // incl. '.'
-	t += skip;
-	olds = s; // *olds == '.', thus we double the dot in the next round
-      }
-    }
-    // *s == \0 here.
-
-    if ( !tlen ) return aStr; // didn't change anything
-
-    // copy last chunk.
-    if ( tlen + s - olds + 1 /* incl. \0 */ >= (int)target.size() ) {
-      target.resize( tlen + s - olds + 1 );
-      t = target.begin() + tlen;
-    }
-    memcpy( t, olds, s - olds + 1 );
-  }
-
-  return target;
-}
-#endif
-
 //-----------------------------------------------------------------------------
 void KMSendProc::statusMsg(const QString& aMsg)
 {
@@ -924,6 +911,8 @@ bool KMSendSendmail::send(KMMessage* aMsg)
   mMailerProc->clearArguments();
   *mMailerProc << mSender->transportInfo()->host;
   *mMailerProc << "-i";
+  *mMailerProc << "-f";
+  *mMailerProc << aMsg->sender().latin1();
 
   if( !aMsg->headerField("X-KMail-Recipients").isEmpty() ) {
     // extended BCC handling to prevent TOs and CCs from seeing
@@ -1079,19 +1068,21 @@ bool KMSendSMTP::send(KMMessage *aMsg)
 
   KURL destination;
 
-  destination.setProtocol((ti->encryption == "SSL") ? "smtps" : "smtp");
+  destination.setProtocol((ti->encryption == "SSL") ? SMTPS_PROTOCOL : SMTP_PROTOCOL);
   destination.setHost(ti->host);
   destination.setPort(ti->port.toUShort());
 
   if (ti->auth)
   {
-    if(ti->user.isEmpty() || ti->pass.isEmpty())
+    if( (ti->user.isEmpty() || ti->passwd().isEmpty()) && 
+      ti->authType != "GSSAPI" )
     {
       bool b = FALSE;
       int result;
 
       KCursorSaver idle(KBusyPtr::idle());
-      result = KIO::PasswordDialog::getNameAndPassword(ti->user, ti->pass,
+      QString passwd = ti->passwd();
+      result = KIO::PasswordDialog::getNameAndPassword(ti->user, passwd,
 	&b, i18n("You need to supply a username and a password to use this "
 	     "SMTP server."), FALSE, QString::null, ti->name, QString::null);
 
@@ -1100,11 +1091,13 @@ bool KMSendSMTP::send(KMMessage *aMsg)
         abort();
         return FALSE;
       }
-      if (int id = KMTransportInfo::findTransport(ti->name))
+      if (int id = KMTransportInfo::findTransport(ti->name)) {
+        ti->setPasswd( passwd );
         ti->writeConfig(id);
+      }
     }
     destination.setUser(ti->user);
-    destination.setPass(ti->pass);
+    destination.setPass(ti->passwd());
   }
 
   if (!mSlave || !mInProcess)
@@ -1121,12 +1114,8 @@ bool KMSendSMTP::send(KMMessage *aMsg)
     return false;
   }
 
-#if KDE_IS_VERSION( 3, 1, 90 )
   // dotstuffing is now done by the slave (see setting of metadata)
   mMessage = aMsg->asSendableString();
-#else
-  mMessage = prepareStr(aMsg->asSendableString(), TRUE);
-#endif
   mMessageLength = mMessage.length();
   mMessageOffset = 0;
 
@@ -1141,9 +1130,7 @@ bool KMSendSMTP::send(KMMessage *aMsg)
 
   if ((mJob = KIO::put(destination, -1, false, false, false)))
   {
-#if KDE_IS_VERSION( 3, 1, 90 )
     mJob->addMetaData( "lf2crlf+dotstuff", "slave" );
-#endif
     KIO::Scheduler::assignJobToSlave(mSlave, mJob);
     connect(mJob, SIGNAL(result(KIO::Job *)), this, SLOT(result(KIO::Job *)));
     connect(mJob, SIGNAL(dataReq(KIO::Job *, QByteArray &)),
