@@ -36,6 +36,7 @@
 #include "kmmimeparttree.h"
 #include <mimelib/utility.h>
 #include <qregexp.h>
+#include <kasciistricmp.h>
 
 /*
   ===========================================================================
@@ -84,7 +85,7 @@ partNode::partNode( DwBodyPart* dwPart, int explicitType, int explicitSubType,
     mType    = explicitType;     // this happens e.g. for the Root Node
     mSubType = explicitSubType;  // representing the _whole_ message
   } else {
-    kdDebug(5006) << "\n        partNode::partNode()      explicitType == DwMime::kTypeUnknown\n" << endl;
+//    kdDebug(5006) << "\n        partNode::partNode()      explicitType == DwMime::kTypeUnknown\n" << endl;
     if(dwPart && dwPart->hasHeaders() && dwPart->Headers().HasContentType()) {
       mType    = (!dwPart->Headers().ContentType().Type())?DwMime::kTypeUnknown:dwPart->Headers().ContentType().Type();
       mSubType = dwPart->Headers().ContentType().Subtype();
@@ -193,7 +194,7 @@ void partNode::buildObjectTree( bool processSiblings )
         //dive into multipart messages
         while( DwMime::kTypeMultipart == curNode->type() ) {
             partNode * newNode = new partNode( curNode->dwPart()->Body().FirstBodyPart() );
-	    curNode->setFirstChild( newNode );
+            curNode->setFirstChild( newNode );
             curNode = newNode;
         }
         // go up in the tree until reaching a node with next
@@ -210,7 +211,7 @@ void partNode::buildObjectTree( bool processSiblings )
         if( curNode && curNode->dwPart() && curNode->dwPart()->Next() ) {
             partNode* nextNode = new partNode( curNode->dwPart()->Next() );
             curNode->setNext( nextNode );
-	    curNode = nextNode;
+            curNode = nextNode;
         } else
             curNode = 0;
     }
@@ -411,6 +412,18 @@ partNode* partNode::findType( int type, int subType, bool deep, bool wide )
         return 0;
 }
 
+partNode* partNode::findNodeForDwPart( DwBodyPart* part )
+{
+    partNode* found = 0;
+    if( kasciistricmp( dwPart()->partId(), part->partId() ) == 0 )
+        return this;
+    if( mChild )
+        found = mChild->findNodeForDwPart( part );
+    if( mNext && !found )
+        found = mNext->findNodeForDwPart( part );
+    return found;
+}
+
 partNode* partNode::findTypeNot( int type, int subType, bool deep, bool wide )
 {
     if(    (mType != DwMime::kTypeUnknown)
@@ -570,7 +583,7 @@ bool partNode::isFirstTextPart() const {
     if ( p->type() == DwMime::kTypeMessage )
       break;
     else
-    root = p;
+      root = p;
   }
   for ( const partNode * n = root ; n ; n = n->next() )
     if ( n->type() == DwMime::kTypeText )
