@@ -2,7 +2,7 @@
     knarticlefactory.cpp
 
     KNode, the KDE newsreader
-    Copyright (c) 1999-2001 the KNode authors.
+    Copyright (c) 1999-2004 the KNode authors.
     See file AUTHORS for details
 
     This program is free software; you can redistribute it and/or modify
@@ -15,11 +15,12 @@
 */
 
 #include <qlayout.h>
+#include <qlabel.h>
+#include <qvbox.h>
 
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kwin.h>
-#include <kseparator.h>
 #include <kapplication.h>
 
 #include "knarticlefactory.h"
@@ -34,8 +35,6 @@
 #include "knnntpaccount.h"
 #include "utilities.h"
 #include "resource.h"
-#include <qlabel.h>
-#include <qpushbutton.h>
 
 
 KNArticleFactory::KNArticleFactory(QObject *p, const char *n)
@@ -57,7 +56,7 @@ void KNArticleFactory::createPosting(KNNntpAccount *a)
     return;
 
   QString sig;
-  KNLocalArticle *art=newArticle(a, sig, knGlobals.cfgManager->postNewsTechnical()->charset());
+  KNLocalArticle *art=newArticle(a, sig, knGlobals.configManager()->postNewsTechnical()->charset());
   if(!art)
     return;
 
@@ -81,7 +80,7 @@ void KNArticleFactory::createPosting(KNGroup *g)
   if (g->useCharset())
     chset = g->defaultCharset();
   else
-    chset = knGlobals.cfgManager->postNewsTechnical()->charset();
+    chset = knGlobals.configManager()->postNewsTechnical()->charset();
 
   QString sig;
   KNLocalArticle *art=newArticle(g, sig, chset);
@@ -109,17 +108,17 @@ void KNArticleFactory::createReply(KNRemoteArticle *a, QString selectedText, boo
   KNGroup *g=static_cast<KNGroup*>(a->collection());
 
   QCString chset;
-  if (knGlobals.cfgManager->postNewsTechnical()->useOwnCharset()) {
+  if (knGlobals.configManager()->postNewsTechnical()->useOwnCharset()) {
     if (g->useCharset())
       chset = g->defaultCharset();
     else
-      chset = knGlobals.cfgManager->postNewsTechnical()->charset();
+      chset = knGlobals.configManager()->postNewsTechnical()->charset();
   } else
-    chset = knGlobals.cfgManager->postNewsTechnical()->findComposerCharset(a->contentType()->charset());
+    chset = knGlobals.configManager()->postNewsTechnical()->findComposerCharset(a->contentType()->charset());
 
   //create new article
   QString sig;
-  KNLocalArticle *art=newArticle(g, sig, chset);
+  KNLocalArticle *art=newArticle(g, sig, chset, true, a);
   if(!art)
     return;
 
@@ -138,12 +137,15 @@ void KNArticleFactory::createReply(KNRemoteArticle *a, QString selectedText, boo
   //newsgroups
   KMime::Headers::FollowUpTo *fup2=a->followUpTo(false);
   if(fup2 && !fup2->isEmpty()) {
-    if(fup2->as7BitString(false).upper()=="POSTER") { //Followup-To: poster
-      if (post)         // warn the user
-        KMessageBox::information(knGlobals.topWidget,i18n("The author has requested a reply by email instead\nof a followup to the newsgroup. (Followup-To: poster)"),
-                                 QString::null,"followupToPosterWarning");
-      art->setDoPost(false);
-      art->setDoMail(true);
+    if( ( fup2->as7BitString(false).upper()=="POSTER" ) ) { //Followup-To: poster
+      if( post && // user wanted to reply by public posting?
+          // ask the user if she wants to ignore this F'up-To: poster
+          ( KMessageBox::Yes != KMessageBox::questionYesNo(knGlobals.topWidget,
+                i18n("The author has requested a reply by email instead\nof a followup to the newsgroup. (Followup-To: poster)\nDo you want to reply in public anyway?")) ))
+      {
+        art->setDoPost(false);
+        art->setDoMail(true);
+      }
       art->newsgroups()->from7BitString(a->newsgroups()->as7BitString(false));
     }
     else
@@ -205,7 +207,7 @@ void KNArticleFactory::createReply(KNRemoteArticle *a, QString selectedText, boo
   //--------------------------- <Body> -----------------------------
 
   // attribution line
-  QString attribution=knGlobals.cfgManager->postNewsComposer()->intro();
+  QString attribution=knGlobals.configManager()->postNewsComposer()->intro();
   QString name(a->from()->name());
   if (name.isEmpty())
     name = QString::fromLatin1(a->from()->email());
@@ -214,18 +216,19 @@ void KNArticleFactory::createReply(KNRemoteArticle *a, QString selectedText, boo
   attribution.replace(QRegExp("%DATE"),KGlobal::locale()->formatDateTime(a->date()->qdt(),false));
   attribution.replace(QRegExp("%MSID"),a->messageID()->asUnicodeString());
   attribution.replace(QRegExp("%GROUP"),g->groupname());
+  attribution.replace(QRegExp("%L"),"\n");
   attribution+="\n\n";
 
   QString quoted=attribution;
   QString notRewraped=QString::null;
   QStringList text;
   QStringList::Iterator line;
-  bool incSig=knGlobals.cfgManager->postNewsComposer()->includeSignature();
+  bool incSig=knGlobals.configManager()->postNewsComposer()->includeSignature();
 
   if (selectedText.isEmpty()) {
     KMime::Content *tc = a->textContent();
     if(tc)
-      tc->decodedText(text, true, knGlobals.cfgManager->readNewsViewer()->removeTrailingNewlines());
+      tc->decodedText(text, true, knGlobals.configManager()->readNewsViewer()->removeTrailingNewlines());
   }
   else
     text = QStringList::split('\n',selectedText,true);
@@ -240,17 +243,17 @@ void KNArticleFactory::createReply(KNRemoteArticle *a, QString selectedText, boo
       quoted+="> "+(*line)+"\n";
   }
 
-  if(knGlobals.cfgManager->postNewsComposer()->rewrap()) {  //rewrap original article
+  if(knGlobals.configManager()->postNewsComposer()->rewrap()) {  //rewrap original article
 
     notRewraped=quoted;     // store the original text in here, the user can request it in the composer
     quoted=attribution;
 
-    quoted += KNHelper::rewrapStringList(text, knGlobals.cfgManager->postNewsComposer()->maxLineLength(), '>', !incSig, false);
+    quoted += KNHelper::rewrapStringList(text, knGlobals.configManager()->postNewsComposer()->maxLineLength(), '>', !incSig, false);
   }
 
   //-------------------------- </Body> -----------------------------
 
-  if (art->doMail() && knGlobals.cfgManager->postNewsTechnical()->useExternalMailer()) {
+  if (art->doMail() && knGlobals.configManager()->postNewsTechnical()->useExternalMailer()) {
     sendMailExternal(address.asUnicodeString(), subject, quoted);
     art->setDoMail(false);
     if (!art->doPost()) {
@@ -274,20 +277,20 @@ void KNArticleFactory::createForward(KNArticle *a)
 
   KMime::Headers::ContentType *ct=a->contentType();
   QCString chset;
-  bool incAtt = ( !knGlobals.cfgManager->postNewsTechnical()->useExternalMailer() &&
+  bool incAtt = ( !knGlobals.configManager()->postNewsTechnical()->useExternalMailer() &&
                   ct->isMultipart() && ct->isSubtype("mixed") &&
                   KMessageBox::Yes == KMessageBox::questionYesNo(knGlobals.topWidget,
                   i18n("This article contains attachments. Do you want them to be forwarded as well?"))
                 );
 
-  if (knGlobals.cfgManager->postNewsTechnical()->useOwnCharset())
-    chset = knGlobals.cfgManager->postNewsTechnical()->charset();
+  if (knGlobals.configManager()->postNewsTechnical()->useOwnCharset())
+    chset = knGlobals.configManager()->postNewsTechnical()->charset();
   else
-    chset = knGlobals.cfgManager->postNewsTechnical()->findComposerCharset(a->contentType()->charset());
+    chset = knGlobals.configManager()->postNewsTechnical()->findComposerCharset(a->contentType()->charset());
 
   //create new article
   QString sig;
-  KNLocalArticle *art=newArticle(knGlobals.grpManager->currentGroup(), sig, chset);
+  KNLocalArticle *art=newArticle(knGlobals.groupManager()->currentGroup(), sig, chset);
   if(!art)
     return;
 
@@ -304,22 +307,22 @@ void KNArticleFactory::createForward(KNArticle *a)
 
   //--------------------------- <Body> -----------------------------
 
-  QString fwd=QString("\n,--------------- %1\n\n").arg(i18n("Forwarded message (begin)"));
+  QString fwd = QString("\n--------------- %1\n\n").arg(i18n("Forwarded message (begin)"));
 
-  fwd+=( i18n(" Subject")+": "+a->subject()->asUnicodeString()+"\n");
-  fwd+=( i18n(" From")+": "+a->from()->asUnicodeString()+"\n");
-  fwd+=( i18n(" Date")+": "+a->date()->asUnicodeString()+"\n");
-  fwd+=( i18n(" Newsgroup")+": "+a->newsgroups()->asUnicodeString()+"\n\n");
+  fwd+=( i18n("Subject") + ": " + a->subject()->asUnicodeString() + "\n" );
+  fwd+=( i18n("From") + ": " + a->from()->asUnicodeString() + "\n" );
+  fwd+=( i18n("Date") + ": " + a->date()->asUnicodeString() + "\n" );
+  fwd+=( i18n("Newsgroup") + ": " + a->newsgroups()->asUnicodeString() + "\n\n" );
 
   KMime::Content *text=a->textContent();
   if(text) {
     QStringList decodedLines;
-    text->decodedText(decodedLines, true, knGlobals.cfgManager->readNewsViewer()->removeTrailingNewlines());
+    text->decodedText( decodedLines, false, false );
     for(QStringList::Iterator it=decodedLines.begin(); it!=decodedLines.end(); ++it)
-      fwd+=" "+(*it)+"\n";
+      fwd += (*it) + "\n";
   }
 
-  fwd+=QString("\n`--------------- %1\n").arg(i18n("Forwarded message (end)"));
+  fwd += QString("\n--------------- %1\n").arg(i18n("Forwarded message (end)"));
 
   //--------------------------- </Body> ----------------------------
 
@@ -338,7 +341,7 @@ void KNArticleFactory::createForward(KNArticle *a)
   //------------------------ </Attachments> ------------------------
 
 
-  if (knGlobals.cfgManager->postNewsTechnical()->useExternalMailer()) {
+  if (knGlobals.configManager()->postNewsTechnical()->useExternalMailer()) {
     sendMailExternal(QString::null, subject, fwd);
     delete art;
     return;
@@ -375,18 +378,18 @@ void KNArticleFactory::createCancel(KNArticle *a)
     nntp=(static_cast<KNGroup*>(a->collection()))->account();
   else {
     if(!nntp)
-      nntp=knGlobals.accManager->first();
+      nntp=knGlobals.accountManager()->first();
     if(!nntp) {
-      KMessageBox::error(knGlobals.topWidget, i18n("You have no valid news accounts configured!"));
+      KMessageBox::error(knGlobals.topWidget, i18n("You have no valid news accounts configured."));
       return;
     }
     KNLocalArticle *la=static_cast<KNLocalArticle*>(a);
     la->setCanceled(true);
     la->updateListItem();
-    nntp=knGlobals.accManager->account(la->serverId());
+    nntp=knGlobals.accountManager()->account(la->serverId());
   }
 
-  grp=knGlobals.grpManager->group(a->newsgroups()->firstGroup(), nntp);
+  grp=knGlobals.groupManager()->group(a->newsgroups()->firstGroup(), nntp);
 
   QString sig;
   KNLocalArticle *art=newArticle(grp, sig, "us-ascii", false);
@@ -450,20 +453,20 @@ void KNArticleFactory::createSupersede(KNArticle *a)
     KNLocalArticle *la=static_cast<KNLocalArticle*>(a);
     la->setCanceled(true);
     la->updateListItem();
-    nntp=knGlobals.accManager->account(la->serverId());
+    nntp=knGlobals.accountManager()->account(la->serverId());
     if(!nntp)
-      nntp=knGlobals.accManager->first();
+      nntp=knGlobals.accountManager()->first();
     if(!nntp) {
-      KMessageBox::error(knGlobals.topWidget, i18n("You have no valid news accounts configured!"));
+      KMessageBox::error(knGlobals.topWidget, i18n("You have no valid news accounts configured."));
       return;
     }
   }
 
-  grp=knGlobals.grpManager->group(a->newsgroups()->firstGroup(), nntp);
+  grp=knGlobals.groupManager()->group(a->newsgroups()->firstGroup(), nntp);
 
   //new article
   QString sig;
-  KNLocalArticle *art=newArticle(grp, sig, knGlobals.cfgManager->postNewsTechnical()->findComposerCharset(a->contentType()->charset()));
+  KNLocalArticle *art=newArticle(grp, sig, knGlobals.configManager()->postNewsTechnical()->findComposerCharset(a->contentType()->charset()));
   if(!art)
     return;
 
@@ -483,7 +486,8 @@ void KNArticleFactory::createSupersede(KNArticle *a)
   art->followUpTo()->from7BitString(a->followUpTo()->as7BitString(false));
 
   //References
-  art->references()->from7BitString(a->references()->as7BitString(false));
+  if ( !a->references()->isEmpty() )
+    art->references()->from7BitString( a->references()->as7BitString(false) );
 
   //Supersedes
   art->supersedes()->from7BitString(a->messageID()->as7BitString(false));
@@ -504,14 +508,14 @@ void KNArticleFactory::createSupersede(KNArticle *a)
 
 void KNArticleFactory::createMail(KMime::Headers::AddressField *address)
 {
-  if (knGlobals.cfgManager->postNewsTechnical()->useExternalMailer()) {
+  if (knGlobals.configManager()->postNewsTechnical()->useExternalMailer()) {
     sendMailExternal(address->asUnicodeString());
     return;
   }
 
   //create new article
   QString sig;
-  KNLocalArticle *art=newArticle(knGlobals.grpManager->currentGroup(), sig, knGlobals.cfgManager->postNewsTechnical()->charset());
+  KNLocalArticle *art=newArticle(knGlobals.groupManager()->currentGroup(), sig, knGlobals.configManager()->postNewsTechnical()->charset());
   if(!art)
     return;
 
@@ -574,26 +578,32 @@ void KNArticleFactory::edit(KNLocalArticle *a)
   }
 
   //find signature
-  KNConfig::Identity *id=knGlobals.cfgManager->identity();
+  KNConfig::Identity *id=knGlobals.configManager()->identity();
 
   if(a->doPost()) {
-    KNNntpAccount *acc=knGlobals.accManager->account(a->serverId());
+    KNNntpAccount *acc=knGlobals.accountManager()->account(a->serverId());
     if(acc) {
       KMime::Headers::Newsgroups *grps=a->newsgroups();
-      KNGroup *grp=knGlobals.grpManager->group(grps->firstGroup(), acc);
-      if (grp && grp->identity() && grp->identity()->hasSignature())
+      KNGroup *grp=knGlobals.groupManager()->group(grps->firstGroup(), acc);
+      if (grp && grp->identity())
         id=grp->identity();
-      else if (acc->identity() && acc->identity()->hasSignature())
+      else if (acc->identity())
         id=acc->identity();
     }
   }
 
   //load article body
   if(!a->hasContent())
-    knGlobals.artManager->loadArticle(a);
+    knGlobals.articleManager()->loadArticle(a);
 
   //open composer
   com=new KNComposer(a, QString::null, id->getSignature());
+  if(id->useSigGenerator() && !id->getSigGeneratorStdErr().isEmpty())
+  KMessageBox::information(knGlobals.topWidget,
+                            i18n("<qt>The signature generator program produced the "
+                                "following output:<br><br>%1</qt>")
+                                .arg(id->getSigGeneratorStdErr()));
+
   c_ompList.append(com);
   connect(com, SIGNAL(composerDone(KNComposer*)), this, SLOT(slotComposerDone(KNComposer*)));
   com->show();
@@ -620,7 +630,7 @@ void KNArticleFactory::sendArticles(KNLocalArticle::List *l, bool now)
   }
 
   if(!now) {
-    knGlobals.artManager->moveIntoFolder(unsent, knGlobals.folManager->outbox());
+    knGlobals.articleManager()->moveIntoFolder(unsent, knGlobals.folderManager()->outbox());
     return;
   }
 
@@ -631,20 +641,20 @@ void KNArticleFactory::sendArticles(KNLocalArticle::List *l, bool now)
       continue;
 
     if(!a->hasContent()) {
-      if(!knGlobals.artManager->loadArticle(a)) {
+      if(!knGlobals.articleManager()->loadArticle(a)) {
         showSendErrorDialog();
-        s_endErrDlg->append(a->subject()->asUnicodeString(), i18n("Unable to load article!"));
+        s_endErrDlg->append(a->subject()->asUnicodeString(), i18n("Unable to load article."));
         continue;
       }
     }
 
     if(a->doPost() && !a->posted()) {
-      ser=knGlobals.accManager->account(a->serverId());
+      ser=knGlobals.accountManager()->account(a->serverId());
       job=new KNJobData(KNJobData::JTpostArticle, this, ser, a);
       emitJob(job);
     }
     else if(a->doMail() && !a->mailed()) {
-      ser=knGlobals.accManager->smtp();
+      ser=knGlobals.accountManager()->smtp();
       job=new KNJobData(KNJobData::JTmail, this, ser, a);
       emitJob(job);
     }
@@ -657,12 +667,12 @@ void KNArticleFactory::sendOutbox()
   KNLocalArticle::List lst;
   KNFolder *ob=0;
 
-  if(!knGlobals.folManager->loadOutbox()) {
-    KMessageBox::error(knGlobals.topWidget, i18n("Unable to load the outbox-folder!"));
+  if(!knGlobals.folderManager()->loadOutbox()) {
+    KMessageBox::error(knGlobals.topWidget, i18n("Unable to load the outbox-folder."));
     return;
   }
 
-  ob=knGlobals.folManager->outbox();
+  ob=knGlobals.folderManager()->outbox();
   for(int i=0; i< ob->length(); i++)
     lst.append(ob->at(i));
 
@@ -720,20 +730,20 @@ void KNArticleFactory::configChanged()
 
 
 void KNArticleFactory::processJob(KNJobData *j)
-{  
+{
   KNLocalArticle *art=static_cast<KNLocalArticle*>(j->data());
   KNLocalArticle::List lst;
   lst.append(art);
-  
+
   if(j->canceled()) {
     delete j;
-    
+
     //sending of this article was canceled => move it to the "Outbox-Folder"
-    if(art->collection()!=knGlobals.folManager->outbox())
-      knGlobals.artManager->moveIntoFolder(lst, knGlobals.folManager->outbox());    
-              
+    if(art->collection()!=knGlobals.folderManager()->outbox())
+      knGlobals.articleManager()->moveIntoFolder(lst, knGlobals.folderManager()->outbox());
+
     KMessageBox::information(knGlobals.topWidget, i18n("You have aborted the posting of articles. The unsent articles are stored in the \"Outbox\" folder."));
-      
+
     return;
   }
 
@@ -743,8 +753,8 @@ void KNArticleFactory::processJob(KNJobData *j)
     delete j; //unlock article
 
     //sending of this article failed => move it to the "Outbox-Folder"
-    if(art->collection()!=knGlobals.folManager->outbox())
-      knGlobals.artManager->moveIntoFolder(lst, knGlobals.folManager->outbox());
+    if(art->collection()!=knGlobals.folderManager()->outbox())
+      knGlobals.articleManager()->moveIntoFolder(lst, knGlobals.folderManager()->outbox());
   }
   else {
 
@@ -771,14 +781,14 @@ void KNArticleFactory::processJob(KNJobData *j)
     };
 
     //article has been sent successfully => move it to the "Sent-folder"
-    knGlobals.artManager->moveIntoFolder(lst, knGlobals.folManager->sent());
+    knGlobals.articleManager()->moveIntoFolder(lst, knGlobals.folderManager()->sent());
   }
 }
 
 
-KNLocalArticle* KNArticleFactory::newArticle(KNCollection *col, QString &sig, QCString defChset, bool withXHeaders)
+KNLocalArticle* KNArticleFactory::newArticle(KNCollection *col, QString &sig, QCString defChset, bool withXHeaders, KNArticle *origPost)
 {
-  KNConfig::PostNewsTechnical *pnt=knGlobals.cfgManager->postNewsTechnical();
+  KNConfig::PostNewsTechnical *pnt=knGlobals.configManager()->postNewsTechnical();
 
   if(pnt->generateMessageID() && pnt->hostname().isEmpty()) {
     KMessageBox::sorry(knGlobals.topWidget, i18n("Please set a hostname for the generation\nof the message-id or disable it."));
@@ -786,22 +796,25 @@ KNLocalArticle* KNArticleFactory::newArticle(KNCollection *col, QString &sig, QC
   }
 
   KNLocalArticle *art=new KNLocalArticle(0);
-  KNConfig::Identity  *grpId=0,
-                      *defId=0,
-                      *accId=0,
-                      *id=0;
+  KNConfig::Identity *tmpId=0, *id=0;
 
   if (col) {
     if (col->type() == KNCollection::CTgroup) {
-      grpId = (static_cast<KNGroup *>(col))->identity();
-      accId = (static_cast<KNGroup *>(col))->account()->identity();
+      id = (static_cast<KNGroup *>(col))->identity();
+      tmpId = (static_cast<KNGroup *>(col))->account()->identity();
     } else
       if (col->type() == KNCollection::CTnntpAccount) {
-        accId = (static_cast<KNNntpAccount *>(col))->identity();
+        id = (static_cast<KNNntpAccount *>(col))->identity();
       }
   }
 
-  defId=knGlobals.cfgManager->identity();
+  // determine active innermost non-empty identity
+  if (!id) {
+    if (tmpId)
+      id = tmpId;
+    else
+      id = knGlobals.configManager()->identity();
+  }
 
   //Message-id
   if(pnt->generateMessageID())
@@ -812,31 +825,24 @@ KNLocalArticle* KNArticleFactory::newArticle(KNCollection *col, QString &sig, QC
   from->setRFC2047Charset(pnt->charset());
 
   //name
-  if(grpId && grpId->hasName())
-    id=grpId;
-  else
-    id=((accId) && accId->hasName())? accId:defId;
   if(id->hasName())
     from->setName(id->name());
 
   //email
-  if(grpId && grpId->hasEmail())
-    id=grpId;
-  else
-    id=((accId) && accId->hasEmail())? accId:defId;
   if(id->hasEmail()&&id->emailIsValid())
     from->setEmail(id->email().latin1());
   else {
-    KMessageBox::sorry(knGlobals.topWidget, i18n("Please enter a valid email address."));
+    if ( id->hasEmail() )
+      KMessageBox::sorry(knGlobals.topWidget,
+	i18n("Please enter a valid email address at the identity tab of the account configuration dialog."));
+    else
+      KMessageBox::sorry(knGlobals.topWidget,
+         i18n("Please enter a valid email address at the identity section of the configuration dialog."));
     delete art;
     return 0;
   }
 
   //Reply-To
-  if(grpId && grpId->hasReplyTo())
-    id=grpId;
-  else
-    id=((accId) && accId->hasReplyTo())? accId:defId;
   if(id->hasReplyTo()) {
     art->replyTo()->fromUnicodeString(id->replyTo(), pnt->charset());
     if (!art->replyTo()->hasEmail())   // the header is invalid => drop it
@@ -844,10 +850,6 @@ KNLocalArticle* KNArticleFactory::newArticle(KNCollection *col, QString &sig, QC
   }
 
   //Mail-Copies-To
-  if(grpId && grpId->hasMailCopiesTo())
-    id=grpId;
-  else
-    id=((accId) && accId->hasMailCopiesTo())? accId:defId;
   if(id->hasMailCopiesTo()) {
     art->mailCopiesTo()->fromUnicodeString(id->mailCopiesTo(), pnt->charset());
     if (!art->mailCopiesTo()->isValid())   // the header is invalid => drop it
@@ -855,10 +857,6 @@ KNLocalArticle* KNArticleFactory::newArticle(KNCollection *col, QString &sig, QC
   }
 
   //Organization
-  if(grpId && grpId->hasOrga())
-    id=grpId;
-  else
-    id=((accId) && accId->hasOrga())? accId:defId;
   if(id->hasOrga())
     art->organization()->fromUnicodeString(id->orga(), pnt->charset());
 
@@ -884,17 +882,33 @@ KNLocalArticle* KNArticleFactory::newArticle(KNCollection *col, QString &sig, QC
   //X-Headers
   if(withXHeaders) {
     KNConfig::XHeaders::Iterator it;
-    for(it=pnt->xHeaders().begin(); it!=pnt->xHeaders().end(); ++it)
-      art->setHeader( new KMime::Headers::Generic( (QCString("X-")+(*it).name()), art, (*it).value(), pnt->charset() ) );
+    for(it=pnt->xHeaders().begin(); it!=pnt->xHeaders().end(); ++it) {
+      QString value = (*it).value();
+      if(origPost) {
+        QString name(origPost->from()->name());
+        if (name.isEmpty())
+          name = QString::fromLatin1(origPost->from()->email());
+        value.replace(QRegExp("%NAME"), name);
+        value.replace(QRegExp("%EMAIL"), QString::fromLatin1(origPost->from()->email()));
+      }
+      else
+        if(value.find("%NAME") != -1 || value.find("%EMAIL") != -1)
+          continue;
+
+      art->setHeader( new KMime::Headers::Generic( (QCString("X-")+(*it).name()), art, value, pnt->charset() ) );
+    }
   }
 
   //Signature
-  if(grpId && grpId->hasSignature())
-    id=grpId;
-  else
-    id=((accId) && accId->hasSignature())? accId:defId;
   if(id->hasSignature())
+  {
     sig=id->getSignature();
+    if(id->useSigGenerator() && !id->getSigGeneratorStdErr().isEmpty())
+      KMessageBox::information(knGlobals.topWidget,
+                               i18n("<qt>The signature generator program produced the "
+                                   "following output:<br><br>%1</qt>")
+                                   .arg(id->getSigGeneratorStdErr()));
+  }
   else
     sig=QString::null;
 
@@ -911,23 +925,23 @@ bool KNArticleFactory::cancelAllowed(KNArticle *a)
     KNLocalArticle *localArt=static_cast<KNLocalArticle*>(a);
 
     if(localArt->doMail() && !localArt->doPost()) {
-      KMessageBox::sorry(knGlobals.topWidget, i18n("Emails cannot be canceled or superseded!"));
+      KMessageBox::sorry(knGlobals.topWidget, i18n("Emails cannot be canceled or superseded."));
       return false;
     }
 
     KMime::Headers::Control *ctrl=localArt->control(false);
     if(ctrl && ctrl->isCancel()) {
-      KMessageBox::sorry(knGlobals.topWidget, i18n("Cancel messages cannot be canceled or superseded!"));
+      KMessageBox::sorry(knGlobals.topWidget, i18n("Cancel messages cannot be canceled or superseded."));
       return false;
     }
 
     if(!localArt->posted()) {
-      KMessageBox::sorry(knGlobals.topWidget, i18n("Only sent articles can be canceled or superseded!"));
+      KMessageBox::sorry(knGlobals.topWidget, i18n("Only sent articles can be canceled or superseded."));
       return false;
     }
 
     if(localArt->canceled()) {
-      KMessageBox::sorry(knGlobals.topWidget, i18n("This article has already been canceled or superseded!"));
+      KMessageBox::sorry(knGlobals.topWidget, i18n("This article has already been canceled or superseded."));
       return false;
     }
 
@@ -935,7 +949,7 @@ bool KNArticleFactory::cancelAllowed(KNArticle *a)
     if(!mid || mid->isEmpty()) {
       KMessageBox::sorry(knGlobals.topWidget, i18n(
 "This article cannot be canceled or superseded,\n\
-because its message-id has not been created by KNode!\n\
+because its message-id has not been created by KNode.\n\
 But you can look for your article in the newsgroup\n\
 and cancel (or supersede) it there."));
       return false;
@@ -947,27 +961,24 @@ and cancel (or supersede) it there."));
 
     KNRemoteArticle *remArt=static_cast<KNRemoteArticle*>(a);
     KNGroup *g=static_cast<KNGroup*>(a->collection());
-    KNConfig::Identity  *defId=knGlobals.cfgManager->identity(),
+    KNConfig::Identity  *defId=knGlobals.configManager()->identity(),
                         *gid=g->identity(),
                         *accId=g->account()->identity();
-    bool ownArticle=true;
+    bool ownArticle = false;
 
-    if(gid && gid->hasName())
-      ownArticle=( gid->name()==remArt->from()->name() );
-    else
-      if (accId && accId->hasName())
-        ownArticle=( accId->name()==remArt->from()->name() );
-      else
-        ownArticle=( defId->name()==remArt->from()->name() );
+    if (gid && gid->hasName())
+      ownArticle |= ( gid->name() == remArt->from()->name() );
+    if (accId && accId->hasName())
+      ownArticle |= ( accId->name() == remArt->from()->name() );
+    ownArticle |= ( defId->name() == remArt->from()->name() );
 
     if(ownArticle) {
+      ownArticle = false;
       if(gid && gid->hasEmail())
-        ownArticle=( gid->email().latin1()==remArt->from()->email() );
-      else
-        if (accId && accId->hasEmail())
-          ownArticle=( accId->email().latin1()==remArt->from()->email() );
-        else
-          ownArticle=( defId->email().latin1()==remArt->from()->email() );
+        ownArticle |= ( gid->email().latin1() == remArt->from()->email() );
+      if (accId && accId->hasEmail())
+        ownArticle |= ( accId->email().latin1() == remArt->from()->email() );
+      ownArticle |= ( defId->email().latin1() == remArt->from()->email() );
     }
 
     if(!ownArticle) {
@@ -991,7 +1002,7 @@ void KNArticleFactory::showSendErrorDialog()
 {
   if(!s_endErrDlg) {
     s_endErrDlg=new KNSendErrorDialog();
-    connect(s_endErrDlg, SIGNAL(dialogDone()), this, SLOT(slotSendErrorDialogDone()));
+    connect(s_endErrDlg, SIGNAL(closeClicked()), this, SLOT(slotSendErrorDialogDone()));
   }
   s_endErrDlg->show();
 }
@@ -1010,6 +1021,8 @@ void KNArticleFactory::slotComposerDone(KNComposer *com)
       if(delCom) {
         if ( com->applyChanges() )
           sendArticles(&lst, true);
+        else
+          delCom = false;
       }
     break;
 
@@ -1018,20 +1031,22 @@ void KNArticleFactory::slotComposerDone(KNComposer *com)
       if(delCom) {
         if ( com->applyChanges() )
           sendArticles(&lst, false);
+        else
+          delCom = false;
       }
     break;
 
     case KNComposer::CRsave :
       if ( com->applyChanges() )
-        knGlobals.artManager->moveIntoFolder(lst, knGlobals.folManager->drafts());
+        knGlobals.articleManager()->moveIntoFolder(lst, knGlobals.folderManager()->drafts());
     break;
 
     case KNComposer::CRdelAsk:
-      delCom=knGlobals.artManager->deleteArticles(lst, true);
+      delCom=knGlobals.articleManager()->deleteArticles(lst, true);
     break;
 
     case KNComposer::CRdel:
-      delCom=knGlobals.artManager->deleteArticles(lst, false);
+      delCom=knGlobals.articleManager()->deleteArticles(lst, false);
     break;
 
     case KNComposer::CRcancel:
@@ -1051,7 +1066,7 @@ void KNArticleFactory::slotComposerDone(KNComposer *com)
 
 void KNArticleFactory::slotSendErrorDialogDone()
 {
-  delete s_endErrDlg;
+  s_endErrDlg->delayedDestruct();
   s_endErrDlg=0;
 }
 
@@ -1059,36 +1074,21 @@ void KNArticleFactory::slotSendErrorDialogDone()
 //======================================================================================================
 
 
-KNSendErrorDialog::KNSendErrorDialog() : QDialog(knGlobals.topWidget, 0, true)
+KNSendErrorDialog::KNSendErrorDialog()
+  : KDialogBase(knGlobals.topWidget, 0, true, i18n("Errors While Sending"), Close, Close, true)
 {
-  p_ixmap=knGlobals.cfgManager->appearance()->icon(KNConfig::Appearance::sendErr);
+  p_ixmap=knGlobals.configManager()->appearance()->icon(KNConfig::Appearance::sendErr);
 
-  QVBoxLayout *topL=new QVBoxLayout(this, 5,5);
+  QVBox *page = makeVBoxMainWidget();
 
-  QLabel *l=new QLabel(QString("<b>%1</b><br>%2").arg(i18n("Errors occurred while sending these articles:"))
-                                                 .arg(i18n("The unsent articles are stored in the \"Outbox\" folder.")), this);
-  topL->addWidget(l);
-
-  j_obs=new KNDialogListBox(true,this);
-  topL->addWidget(j_obs, 1);
-
-  e_rror=new QLabel(this);
-  topL->addSpacing(5);
-  topL->addWidget(e_rror);
-
-  KSeparator *sep=new KSeparator(this);
-  topL->addSpacing(10);
-  topL->addWidget(sep);   
-
-  c_loseBtn=new QPushButton(i18n("&Close"), this);
-  c_loseBtn->setDefault(true);
-  topL->addWidget(c_loseBtn, 0, Qt::AlignRight);
+  new QLabel(QString("<b>%1</b><br>%2").arg(i18n("Errors occurred while sending these articles:"))
+                                       .arg(i18n("The unsent articles are stored in the \"Outbox\" folder.")), page);
+  j_obs=new KNDialogListBox(true, page);
+  e_rror=new QLabel(QString::null, page);
 
   connect(j_obs, SIGNAL(highlighted(int)), this, SLOT(slotHighlighted(int)));
-  connect(c_loseBtn, SIGNAL(clicked()), this, SLOT(slotCloseBtnClicked()));
 
-  setCaption(kapp->makeStdCaption(i18n("Errors While Sending")));
-  KNHelper::restoreWindowSize("sendDlg", this, sizeHint());
+  KNHelper::restoreWindowSize("sendDlg", this, QSize(320,250));
 }
 
 
@@ -1116,27 +1116,7 @@ void KNSendErrorDialog::slotHighlighted(int idx)
   }
 }
 
-
-void KNSendErrorDialog::slotCloseBtnClicked()
-{
-  emit dialogDone();
-}
-
-
-void KNSendErrorDialog::keyPressEvent(QKeyEvent *e)
-{
-  if ((e->key()==Key_Enter)||(e->key()==Key_Return)||(e->key()==Key_Escape))
-    emit dialogDone();
-  else
-    QDialog::keyPressEvent(e);
-}
-
-
-void KNSendErrorDialog::closeEvent(QCloseEvent *e)
-{
-  e->accept();
-  emit dialogDone();
-}
-
 //-------------------------------
 #include "knarticlefactory.moc"
+
+// kate: space-indent on; indent-width 2;
