@@ -16,14 +16,18 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <cstdlib>
 #include <unistd.h>
 
+#include <qaccel.h>
 #include <qclipboard.h>
 #include <qdir.h>
 #include <qpaintdevicemetrics.h>
 #include <qpainter.h>
 #include <qtimer.h>
 #include <qurl.h>
+#include <qbuffer.h>
+#include <qbitmap.h>
 
 #include <kprinter.h>
 #include <kiconloader.h>
@@ -39,6 +43,8 @@
 #include <kpgpblock.h>
 #include <kstringhandler.h>
 
+#include <libkdepim/kxface.h>
+
 #include "resource.h"
 #include "knarticlewidget.h"
 #include "knglobals.h"
@@ -57,6 +63,7 @@
 #include <kstandarddirs.h>
 #include <kbookmarkmanager.h>
 #include <kaddrbook.h>
+#include <ktempfile.h>
 
 #define PUP_OPEN    1000
 #define PUP_SAVE    2000
@@ -79,6 +86,8 @@ KNSourceViewWindow::KNSourceViewWindow(const QString &htmlCode)
   : KTextBrowser(0)
 {
   setWFlags(WType_TopLevel | WDestructiveClose);
+  QAccel *accel = new QAccel( this, "browser close-accel" );
+  accel->connectItem( accel->insertItem( Qt::Key_Escape ), this , SLOT( close() ));
   KNConfig::Appearance *app=knGlobals.configManager()->appearance();
 
   setCaption(kapp->makeStdCaption(i18n("Article Source")));
@@ -94,6 +103,7 @@ KNSourceViewWindow::KNSourceViewWindow(const QString &htmlCode)
   style->setFontUnderline(app->articleFixedFont().underline());
   style->setFontWeight(app->articleFixedFont().weight());
   style->setFontItalic(app->articleFixedFont().italic());
+  style->setColor( app->textColor() );
 
   setText(QString("<qt><txt>%1</txt></qt>").arg(htmlCode));
   KNHelper::restoreWindowSize("sourceWindow", this, QSize(500,300));
@@ -134,10 +144,10 @@ QByteArray KNMimeSource::encodedData(const char *) const
 //=============================================================================================================
 
 
-KNArticleWidget::KNArticleWidget(KActionCollection* actColl, KXMLGUIClient* guiClient, 
+KNArticleWidget::KNArticleWidget(KActionCollection* actColl, KXMLGUIClient* guiClient,
     QWidget *parent, const char *name )
     : KTextBrowser(parent, name), a_rticle(0), a_tt(0), h_tmlDone(false),
-      emuKMail(false), f_inddialog(0), a_ctions(actColl), mGuiClient(guiClient)
+      f_inddialog(0), a_ctions(actColl), mGuiClient(guiClient)
 {
   instances()->append(this);
   setNotifyClick( true );
@@ -186,9 +196,7 @@ KNArticleWidget::KNArticleWidget(KActionCollection* actColl, KXMLGUIClient* guiC
                           SLOT(slotVerify()), a_ctions, "article_verify");
   a_ctToggleFullHdrs    = new KToggleAction(i18n("Show &All Headers"), "text_block", 0 , this,
                           SLOT(slotToggleFullHdrs()), a_ctions, "view_showAllHdrs");
-#if KDE_IS_VERSION(3,2,90)
   a_ctToggleFullHdrs->setCheckedState(i18n("Hide &All Headers"));
-#endif
   a_ctToggleFullHdrs->setChecked(knGlobals.configManager()->readNewsViewer()->showFullHdrs());
   a_ctToggleRot13       = new KToggleAction(i18n("&Unscramble (Rot 13)"), "decrypted", 0 , this,
                           SLOT(slotToggleRot13()), a_ctions, "view_rot13");
@@ -221,6 +229,10 @@ KNArticleWidget::KNArticleWidget(KActionCollection* actColl, KXMLGUIClient* guiC
   r_ot13=false;
   a_ctToggleRot13->setChecked(false);
   applyConfig();
+
+  t_mpFile = new KTempFile(locateLocal("tmp", "knode"), ".png");
+  t_mpFile->close();
+  t_mpFile->setAutoDelete(true);
 }
 
 
@@ -241,6 +253,7 @@ KNArticleWidget::~KNArticleWidget()
   delete t_imer;
   delete f_inddialog;
   delete u_mailtoPopup;
+  delete t_mpFile;
 }
 
 void KNArticleWidget::setText( const QString& text, const QString& context )
@@ -364,18 +377,6 @@ void KNArticleWidget::keyPressEvent(QKeyEvent *e)
     case Key_Next:
       scrollBy( 0, visibleHeight()-offs);
       break;
-    case Key_Left:
-      if (emuKMail)
-        emit(keyLeftPressed());
-      else
-        QTextBrowser::keyPressEvent(e);
-      break;
-    case Key_Right:
-      if (emuKMail)
-        emit(keyRightPressed());
-      else
-        QTextBrowser::keyPressEvent(e);
-      break;
     default:
       QTextBrowser::keyPressEvent(e);
   }
@@ -479,7 +480,6 @@ void KNArticleWidget::applyConfig()
   if(!knGlobals.configManager()->readNewsGeneral()->autoMark())
     t_imer->stop();
 
-  emuKMail = ((this==knGlobals.artWidget) && knGlobals.configManager()->readNewsNavigation()->emulateKMail());
   updateContents();
 }
 
@@ -1085,6 +1085,22 @@ void KNArticleWidget::createHtmlPage()
 
   QString headerHtml;
   int headerLines=0;
+  int headerCols=3;
+
+  QString xfhead;
+  KMime::Headers::Base *temp = a_rticle->getHeaderByType("X-Face");
+  if (temp)
+    xfhead = temp->asUnicodeString();
+  QString xface = "";
+  if ( !xfhead.isEmpty() )
+  {
+    KPIM::KXFace xf;
+    xf.toImage(xfhead).save(t_mpFile->name(), "PNG");
+    xface = QString::fromLatin1(
+        "<td rowspan=\"4\" width=\"48\" height=\"48\"><img border=\"1\" src=\"%1\"></td>")
+        .arg(t_mpFile->name());
+    ++headerCols;
+  }
 
   if(a_ctToggleFullHdrs->isChecked()) {
     QCString head = a_rticle->head();
@@ -1095,9 +1111,11 @@ void KNArticleWidget::createHtmlPage()
       if (header) {
         if (headerLines > 0)
           headerHtml += "<tr>";
-        headerHtml+=QString("<td align=right><articlefont><b>%1</b></articlefont></td><td width=\"100%\"><articlefont>%2</articlefont></td></tr>")
+        headerHtml+=QString("<td align=right><articlefont><b>%1</b></articlefont></td><td width=\"100%\"><articlefont>%2</articlefont></td>%3</tr>")
                       .arg(toHtmlString(header->type())+":")
-                      .arg(toHtmlString(header->asUnicodeString(),true, false, false, true));
+                      .arg(toHtmlString(header->asUnicodeString(),true, false, false, true))
+                      .arg(xface);
+        xface = "";
         delete header;
         headerLines++;
       }
@@ -1134,7 +1152,8 @@ void KNArticleWidget::createHtmlPage()
       } else
         headerHtml+=toHtmlString(hb->asUnicodeString(),true, false, false, true);
 
-      headerHtml += dh->headerCloseTag()+"</articlefont></td></tr>";
+      headerHtml += dh->headerCloseTag()+"</articlefont></td>"+xface+"</tr>";
+      xface = "";
       headerLines++;
     }
   }
@@ -1147,8 +1166,8 @@ void KNArticleWidget::createHtmlPage()
     if (headerLines > 0) {
       html+=QString("<tr><td width=\"40\" bgcolor=\"%1\" rowspan=\"%2\"></td>"+headerHtml)
            .arg(app->headerDecoHexcode()).arg(headerLines);
-      html+=QString("<tr><td colspan=\"3\" bgcolor=\"%1\"><articlefont>")
-            .arg(app->headerDecoHexcode());
+      html+=QString("<tr><td colspan=\"%1\" bgcolor=\"%2\"><articlefont>")
+          .arg(headerCols).arg(app->headerDecoHexcode());
     } else {
       html+=QString("<tr><td bgcolor=\"%1\"><articlefont>")
           .arg(app->headerDecoHexcode());
@@ -1422,7 +1441,7 @@ void KNArticleWidget::createHtmlPage()
         if(rnv->showAttachmentsInline() && inlinePossible(var)) {
           html+="<tr><td colspan=3>";
           if(ct->isImage()) { //image
-            path=QString::number(attCnt)+"_"+QString::number((int)(a_rticle));
+            path=QString::number(attCnt)+"_"+QString::number((unsigned long)(a_rticle));
             f_actory->setData(path,new KNMimeSource(var->decodedContent(),ct->mimeType()));
             html+=QString("<a href=\"internal://att=%1\"><img src=\"%2\"></a>").arg(attCnt).arg(path);
           }
