@@ -2,8 +2,8 @@
     This file is part of KDE Kontact.
 
     Copyright (c) 2001 Matthias Hoelzer-Kluepfel <mhk@kde.org>
-    Copyright (c) 2002-2003 Daniel Molkentin <molkentin@kde.org>
-    Copyright (c) 2003 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (c) 2002-2005 Daniel Molkentin <molkentin@kde.org>
+    Copyright (c) 2003-2005 Cornelius Schumacher <schumacher@kde.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
-
 #include <qcombobox.h>
 #include <qhbox.h>
 #include <qimage.h>
@@ -56,33 +55,37 @@
 #include <ktrader.h>
 #include <ksettings/componentsdialog.h>
 #include <kstringhandler.h>
-
-#include <infoextension.h>
+#include <krsqueezedtextlabel.h>
+#include <khtml_part.h>
+#include <khtmlview.h>
+#include <libkdepim/kfileio.h>
+#include <kcursor.h>
+#include <krun.h>
+#include <kaboutdata.h>
+#include <kmenubar.h>
 
 #include "aboutdialog.h"
 #include "iconsidepane.h"
 #include "mainwindow.h"
 #include "plugin.h"
 #include "prefs.h"
-#include "sidepane.h"
 #include "progressdialog.h"
 #include "statusbarprogresswidget.h"
 #include "broadcaststatus.h"
-#include "splash.h"
 
 using namespace Kontact;
 
-MainWindow::MainWindow(Kontact::Splash *splash)
-  : Kontact::Core(), mTopWidget( 0 ), mHeaderText( 0 ), mHeaderPixmap( 0 ), mSplitter( 0 ),
-    mSplash( splash ), mCurrentPlugin( 0 ), mLastInfoExtension( 0 ), mAboutDialog( 0 ),
-    mReallyClose( false )
+MainWindow::MainWindow()
+  : Kontact::Core(), mTopWidget( 0 ), mSplitter( 0 ),
+    mCurrentPlugin( 0 ), mAboutDialog( 0 ), mReallyClose( false ),
+    mStartupCompleted( false )
 {
   // Set this to be the group leader for all subdialogs - this means
   // modal subdialogs will only affect this dialog, not the other windows
   setWFlags( getWFlags() | WGroupLeader );
 
   initGUI();
-  QTimer::singleShot( 0, this, SLOT(initObject()) );
+  initObject();
 }
 
 void MainWindow::initGUI()
@@ -104,7 +107,6 @@ void MainWindow::initGUI()
 
   resize( 700, 520 ); // initial size to prevent a scrollbar in sidepane
   setAutoSaveSettings();
-
 }
 
 
@@ -115,15 +117,9 @@ void MainWindow::initObject()
       QString( "[X-KDE-KontactPluginVersion] == %1" ).arg( KONTACT_PLUGIN_VERSION ) );
   mPluginInfos = KPluginInfo::fromServices( offers, Prefs::self()->config(), "Plugins" );
 
-  QProgressBar *bar = mSplash->progressBar();
-  int count = 3;
-  count += mPluginInfos.count();
-  bar->setTotalSteps( count );
-
   KPluginInfo::List::Iterator it;
   for ( it = mPluginInfos.begin(); it != mPluginInfos.end(); ++it ) {
     ( *it )->load();
-    bar->setProgress( bar->progress() + 1 );
   }
 
 
@@ -134,29 +130,19 @@ void MainWindow::initObject()
 
   loadPlugins();
 
-  bar->setProgress( bar->progress() + 1 );
-
   if ( mSidePane )
-    mSidePane->updatePlugins();
-
-  // flush paint events
-  kapp->processEvents();
-
-  bar->setProgress( bar->progress() + 1 );
+  {
+      mSidePane->updatePlugins();
+      plugActionList( "navigator_actionlist", mSidePane->actions() );
+  }
 
   KSettings::Dispatcher::self()->registerInstance( instance(), this,
                                                    SLOT( updateConfig() ) );
 
   loadSettings();
 
-  bar->setProgress( bar->progress() + 1 );
-
   statusBar()->show();
 
-  // we are done - remove splash
-  delete mSplash;
-  mSplash=0;
-
   showTip( false );
 
   // done initializing
@@ -166,9 +152,17 @@ void MainWindow::initObject()
            this, SLOT( slotShowStatusMsg( const QString&  ) ) );
 
   // launch commandline specified module if any
-  // TODO: GUI Option
   activatePluginModule();
 
+  // keep this above the lastVersionSeen check!
+  mStartupCompleted = true;
+
+  if ( Prefs::lastVersionSeen() == kapp->aboutData()->version() ) {
+    selectPlugin( mCurrentPlugin );
+  }
+
+  paintAboutScreen( introductionString() );
+  Prefs::setLastVersionSeen( kapp->aboutData()->version() );
 }
 
 MainWindow::~MainWindow()
@@ -185,15 +179,15 @@ MainWindow::~MainWindow()
   Prefs::self()->writeConfig();
 }
 
-void MainWindow::activePluginModule( const QString &_module )
+void MainWindow::setActivePluginModule( const QString &module )
 {
-  mActiveModule = _module;
+  mActiveModule = module;
+  activatePluginModule();
 }
 
 void MainWindow::activatePluginModule()
 {
-  if ( !mActiveModule.isEmpty() )
-  {
+  if ( !mActiveModule.isEmpty() ) {
     PluginList::ConstIterator end = mPlugins.end();
     for ( PluginList::ConstIterator it = mPlugins.begin(); it != end; ++it )
       if ( ( *it )->identifier().contains( mActiveModule ) ) {
@@ -205,31 +199,22 @@ void MainWindow::activatePluginModule()
 
 void MainWindow::initWidgets()
 {
-  QHBox *topWidget = new QHBox( this );
-  topWidget->setFrameStyle( QFrame::Panel | QFrame::Sunken );
-
-  mTopWidget = topWidget;
-
+  // includes sidebar and part stack
+  mTopWidget = new QHBox( this );
+  mTopWidget->setFrameStyle( QFrame::Panel | QFrame::Sunken );
   setCentralWidget( mTopWidget );
 
-  mSidePaneType = Prefs::self()->mSidePaneType;
-
   QHBox *mBox = 0;
-
-  if ( mSidePaneType == Prefs::SidePaneBars ) {
-    mSplitter = new QSplitter( mTopWidget );
-    mSidePane = new SidePane( this, mSplitter );
-    mSidePane->setSizePolicy( QSizePolicy( QSizePolicy::Maximum,
-                                           QSizePolicy::Preferred ) );
-    mSplitter->setResizeMode( mSidePane, QSplitter::KeepSize );
-  } else {
-    mSplitter = 0;
-    mBox = new QHBox( mTopWidget );
-    mSidePane = new IconSidePane( this, mBox );
-    mSidePane->setSizePolicy( QSizePolicy( QSizePolicy::Maximum,
-                                           QSizePolicy::Preferred ) );
-  }
-
+  mSplitter = new QSplitter( mTopWidget );
+  mBox = new QHBox( mTopWidget );
+  mSidePane = new IconSidePane( this, mSplitter );
+  mSidePane->setSizePolicy( QSizePolicy( QSizePolicy::Maximum,
+                                         QSizePolicy::Preferred ) );
+  // donÄt occupy screen estate on load
+  QValueList<int> sizes;
+  sizes << 0;
+  mSplitter->setSizes(sizes);
+ 
   mSidePane->setActionCollection( actionCollection() );
 
   connect( mSidePane, SIGNAL( pluginSelected( Kontact::Plugin * ) ),
@@ -242,13 +227,15 @@ void MainWindow::initWidgets()
     vBox = new QVBox( mBox );
   }
 
-  initHeaderWidget( vBox );
-  if ( mSidePaneType != Prefs::SidePaneBars )
-    mHeaderFrame->hide();
-
   vBox->setSpacing( 0 );
 
-  mStack = new QWidgetStack( vBox );
+  mPartsStack = new QWidgetStack( vBox );
+  initAboutScreen();
+
+  QString loading = i18n( "<h2 style='text-align:center; margin-top: 0px; margin-bottom: 0px'>%1</h2>" )
+                    .arg( i18n("Loading Kontact...") );
+
+  paintAboutScreen( loading );
 
   /* Create a progress dialog and hide it. */
   KPIM::ProgressDialog *progressDialog = new KPIM::ProgressDialog( statusBar(), this );
@@ -256,18 +243,64 @@ void MainWindow::initWidgets()
 
   mLittleProgress = new KPIM::StatusbarProgressWidget( progressDialog, statusBar() );
 
-  mStatusMsgLabel = new KStatusBarLabel( i18n( " Initializing..." ), 1, statusBar() );
+  mStatusMsgLabel = new KRSqueezedTextLabel( i18n( " Initializing..." ), statusBar() );
   mStatusMsgLabel->setAlignment( AlignLeft | AlignVCenter );
 
-  statusBar()->addWidget( mStatusMsgLabel, 1 , false );
+  statusBar()->addWidget( mStatusMsgLabel, 10 , false );
   statusBar()->addWidget( mLittleProgress, 0 , true );
   mLittleProgress->show();
 }
 
+
+void MainWindow::paintAboutScreen( const QString& msg )
+{
+  QString location = locate( "data", "kontact/about/main.html" );
+  QString content = KPIM::kFileToString( location );
+  content = content.arg( locate( "data", "libkdepim/about/kde_infopage.css" ) );
+  if ( kapp->reverseLayout() )
+    content = content.arg( "@import \"%1\";" ).arg( locate( "data", "libkdepim/about/kde_infopage_rtl.css" ) );
+  else
+    content = content.arg( "" );
+
+  mIntroPart->begin( KURL( location ) );
+
+  QString appName( i18n( "KDE Kontact" ) );
+  QString catchPhrase( i18n( "Get Organized!" ) );
+  QString quickDescription( i18n( "The KDE Personal Information Management Suite" ) );
+
+  mIntroPart->write( content.arg( QFont().pointSize() + 2 ).arg( appName )
+      .arg( catchPhrase ).arg( quickDescription ).arg( msg ) );
+  mIntroPart->end();
+}
+
+void MainWindow::initAboutScreen()
+{
+  QHBox *introbox = new QHBox( mPartsStack );
+  mPartsStack->addWidget( introbox );
+  mPartsStack->raiseWidget( introbox );
+  mIntroPart = new KHTMLPart( introbox );
+  mIntroPart->widget()->setFocusPolicy( WheelFocus );
+  // Let's better be paranoid and disable plugins (it defaults to enabled):
+  mIntroPart->setPluginsEnabled( false );
+  mIntroPart->setJScriptEnabled( false ); // just make this explicit
+  mIntroPart->setJavaEnabled( false );    // just make this explicit
+  mIntroPart->setMetaRefreshEnabled( false );
+  mIntroPart->setURLCursor( KCursor::handCursor() );
+  mIntroPart->view()->setLineWidth( 0 );
+
+  connect( mIntroPart->browserExtension(),
+           SIGNAL( openURLRequest( const KURL&, const KParts::URLArgs& ) ),
+           SLOT( slotOpenUrl( const KURL& ) ) );
+
+  connect( mIntroPart->browserExtension(),
+           SIGNAL( createNewWindow( const KURL&, const KParts::URLArgs& ) ),
+           SLOT( slotOpenUrl( const KURL& ) ) );
+}
+
 void MainWindow::setupActions()
 {
   KStdAction::quit( this, SLOT( slotQuit() ), actionCollection() );
-  mNewActions = new KToolBarPopupAction( KGuiItem(i18n( "New" ), ""),
+  mNewActions = new KToolBarPopupAction( KGuiItem( i18n( "New" ), "" ),
                                          KShortcut(), this, SLOT( slotNewClicked() ),
                                          actionCollection(), "action_new" );
 
@@ -278,55 +311,24 @@ void MainWindow::setupActions()
   new KAction( i18n( "Configure Kontact..." ), "configure", 0, this, SLOT( slotPreferences() ),
                actionCollection(), "settings_configure_kontact" );
 
+  new KAction( i18n( "&Kontact Introduction" ), 0, this, SLOT( slotShowIntroduction() ),
+               actionCollection(), "help_introduction" );
   new KAction( i18n( "&Tip of the Day" ), 0, this, SLOT( slotShowTip() ),
                actionCollection(), "help_tipofday" );
   new KAction( i18n( "&Request Feature..." ), 0, this, SLOT( slotRequestFeature() ),
                actionCollection(), "help_requestfeature" );
-
 }
 
-void MainWindow::initHeaderWidget( QVBox *vBox )
+bool MainWindow::isPluginLoaded( const KPluginInfo *info )
 {
-  // Initiate the headerWidget
-  mHeaderFrame = new QHBox( vBox );
-  mHeaderFrame->setSizePolicy( QSizePolicy::MinimumExpanding,
-                               QSizePolicy::Maximum );
-  mHeaderFrame->setSpacing( 0 );
-  mHeaderFrame->setFixedHeight( 22 );
-
-  mHeaderText = new QLabel( mHeaderFrame );
-  mHeaderText->setSizePolicy( QSizePolicy::MinimumExpanding,
-                              QSizePolicy::Preferred );
-  mHeaderText->setPaletteForegroundColor( colorGroup().light() );
-  mHeaderText->setPaletteBackgroundColor( colorGroup().dark() );
-
-  mHeaderPixmap = new QLabel( mHeaderFrame );
-  mHeaderPixmap->setSizePolicy( QSizePolicy::Maximum,
-                                QSizePolicy::Preferred );
-  mHeaderPixmap->setAlignment( AlignRight|AlignVCenter );
-  mHeaderPixmap->setPaletteBackgroundColor( colorGroup().dark() );
-
-  connect( this, SIGNAL( textChanged( const QString& ) ),
-           this, SLOT( setHeaderText( const QString& ) ) );
-  connect( this, SIGNAL( iconChanged( const QPixmap& ) ),
-           this, SLOT( setHeaderPixmap( const QPixmap& ) ) );
-
-  QFont fnt( mSidePane->font() );
-  fnt.setBold( true );
-  fnt.setPointSize( mSidePane->font().pointSize() + 3 );
-  mHeaderText->setFont( fnt );
-}
-
-bool MainWindow::isPluginLoaded( const KPluginInfo * info )
-{
-  return ( pluginFromInfo( info ) != 0 );
+  return (pluginFromInfo( info ) != 0);
 }
 
 Plugin *MainWindow::pluginFromInfo( const KPluginInfo *info )
 {
   PluginList::ConstIterator end = mPlugins.end();
   for ( PluginList::ConstIterator it = mPlugins.begin(); it != end; ++it )
-    if ( ( *it )->identifier() == info->pluginName() )
+    if ( (*it)->identifier() == info->pluginName() )
       return *it;
 
   return 0;
@@ -340,7 +342,7 @@ void MainWindow::loadPlugins()
   uint i;
   KPluginInfo::List::ConstIterator it;
   for ( it = mPluginInfos.begin(); it != mPluginInfos.end(); ++it ) {
-    if ( ! ( *it )->isPluginEnabled() )
+    if ( !(*it)->isPluginEnabled() )
       continue;
     if ( isPluginLoaded( *it ) ) {
       Plugin *plugin = pluginFromInfo( *it );
@@ -349,21 +351,22 @@ void MainWindow::loadPlugins()
       continue;
     }
 
-    kdDebug(5600) << "Loading Plugin: " << ( *it )->name() << endl;
+    kdDebug(5600) << "Loading Plugin: " << (*it)->name() << endl;
     Kontact::Plugin *plugin =
       KParts::ComponentFactory::createInstanceFromService<Kontact::Plugin>(
-          ( *it )->service(), this );
+          (*it)->service(), this );
 
     if ( !plugin )
       continue;
 
-    plugin->setIdentifier( ( *it )->pluginName() );
-    plugin->setTitle( ( *it )->name() );
-    plugin->setIcon( ( *it )->icon() );
+    plugin->setIdentifier( (*it)->pluginName() );
+    plugin->setTitle( (*it)->name() );
+    plugin->setIcon( (*it)->icon() );
 
-    QVariant libNameProp = ( *it )->property( "X-KDE-KontactPartLibraryName" );
-    QVariant exeNameProp = ( *it )->property( "X-KDE-KontactPartExecutableName" );
-    QVariant loadOnStart = ( *it )->property( "X-KDE-KontactPartLoadOnStart" );
+    QVariant libNameProp = (*it)->property( "X-KDE-KontactPartLibraryName" );
+    QVariant exeNameProp = (*it)->property( "X-KDE-KontactPartExecutableName" );
+    QVariant loadOnStart = (*it)->property( "X-KDE-KontactPartLoadOnStart" );
+    QVariant hasPartProp = (*it)->property( "X-KDE-KontactPluginHasPart" );
 
     if ( !loadOnStart.isNull() && loadOnStart.toBool() )
       mDelayedPreload.append( plugin );
@@ -372,10 +375,13 @@ void MainWindow::loadPlugins()
 
     plugin->setPartLibraryName( libNameProp.toString().utf8() );
     plugin->setExecutableName( exeNameProp.toString() );
+    if ( hasPartProp.isValid() )
+      plugin->setShowInSideBar( hasPartProp.toBool() );
 
     for ( i = 0; i < plugins.count(); ++i ) {
       Plugin *p = plugins.at( i );
-      if ( plugin->weight() < p->weight() ) break;
+      if ( plugin->weight() < p->weight() )
+        break;
     }
 
     plugins.insert( i, plugin );
@@ -395,8 +401,6 @@ void MainWindow::loadPlugins()
     addPlugin( plugin );
   }
 
-  mLastInfoExtension = 0;
-
   mNewActions->setEnabled( mPlugins.size() != 0 );
 }
 
@@ -405,12 +409,12 @@ void MainWindow::unloadPlugins()
   KPluginInfo::List::ConstIterator end = mPluginInfos.end();
   KPluginInfo::List::ConstIterator it;
   for ( it = mPluginInfos.begin(); it != end; ++it ) {
-    if ( ! ( *it )->isPluginEnabled() )
+    if ( !(*it)->isPluginEnabled() )
       removePlugin( *it );
   }
 }
 
-bool MainWindow::removePlugin( const KPluginInfo * info )
+bool MainWindow::removePlugin( const KPluginInfo *info )
 {
   PluginList::Iterator end = mPlugins.end();
   for ( PluginList::Iterator it = mPlugins.begin(); it != end; ++it )
@@ -449,10 +453,13 @@ void MainWindow::addPlugin( Kontact::Plugin *plugin )
   insertChildClient( plugin );
 }
 
-void MainWindow::partLoaded( Kontact::Plugin * /*plugin*/, KParts::Part *part )
+void MainWindow::partLoaded( Kontact::Plugin*, KParts::ReadOnlyPart *part )
 {
-  if ( part->widget() )
-    mStack->addWidget( part->widget(), 0 );
+  // See if we have this part already (e.g. due to two plugins sharing it)
+  if ( mPartsStack->id( part->widget() ) != -1 )
+    return;
+
+  mPartsStack->addWidget( part->widget() );
 
   mPartManager->addPart( part, false );
   // Workaround for KParts misbehavior: addPart calls show!
@@ -466,35 +473,10 @@ void MainWindow::slotActivePartChanged( KParts::Part *part )
     return;
   }
 
-  if ( mLastInfoExtension ) {
-    disconnect( mLastInfoExtension, SIGNAL( textChanged( const QString& ) ),
-                this, SLOT( setHeaderText( const QString& ) ) );
-    disconnect( mLastInfoExtension, SIGNAL( iconChanged( const QPixmap& ) ),
-                this, SLOT( setHeaderPixmap( const QPixmap& ) ) );
-  }
-
   kdDebug(5600) << "Part activated: " << part << " with stack id. "
-      << mStack->id( part->widget() )<< endl;
-  QObjectList *l = part->queryList( "KParts::InfoExtension" );
-  KParts::InfoExtension *ie = 0;
-  if ( l )
-    ie = static_cast<KParts::InfoExtension*>( l->first() );
-  delete l;
+      << mPartsStack->id( part->widget() )<< endl;
 
-  if ( ie ) {
-    connect( ie, SIGNAL( textChanged( const QString& ) ),
-             SLOT( setHeaderText( const QString& ) ) );
-    connect( ie, SIGNAL( iconChanged( const QPixmap& ) ),
-             SLOT( setHeaderPixmap( const QPixmap& ) ) );
-  }
-
-  mLastInfoExtension = ie;
-
-  InfoExtData data = mInfoExtCache[ ie ];
-  setHeaderPixmap( data.pixmap );
-  setHeaderText( data.text );
-
-  createGUI( part );
+  //createGUI( part ); // moved to selectPlugin()
 
   statusBar()->clear();
 }
@@ -523,24 +505,39 @@ void MainWindow::selectPlugin( Kontact::Plugin *plugin )
 
   if ( plugin->isRunningStandalone() ) {
     statusBar()->message( i18n( "Application is running standalone. Foregrounding..." ), 1000 );
+    mSidePane->indicateForegrunding( plugin );
     plugin->bringToForeground();
     return;
   }
 
   KApplication::setOverrideCursor( QCursor( Qt::WaitCursor ) );
 
-  if ( mSidePane )
-    mSidePane->selectPlugin( plugin );
-
   KParts::Part *part = plugin->part();
 
   if ( !part ) {
-    KMessageBox::error( this, i18n( "Cannot load part for %1." )
-                              .arg( plugin->title() ) );
     KApplication::restoreOverrideCursor();
+    KMessageBox::error( this, i18n( "Cannot load part for %1." )
+                              .arg( plugin->title() )
+                        + "\n" + lastErrorMessage() );
     return;
   }
 
+  // store old focus widget
+  QWidget *focusWidget = kapp->focusWidget();
+  if ( mCurrentPlugin && focusWidget ) {
+    // save the focus widget only when it belongs to the activated part
+    QWidget *parent = focusWidget->parentWidget();
+    while ( parent ) {
+      if ( parent == mCurrentPlugin->part()->widget() )
+        mFocusWidgets.insert( mCurrentPlugin->identifier(), QGuardedPtr<QWidget>( focusWidget ) );
+
+      parent = parent->parentWidget();
+    }
+  }
+
+  if ( mSidePane )
+    mSidePane->selectPlugin( plugin );
+
   plugin->select();
 
   mPartManager->setActivePart( part );
@@ -548,12 +545,25 @@ void MainWindow::selectPlugin( Kontact::Plugin *plugin )
   Q_ASSERT( view );
 
   if ( view ) {
-    mStack->raiseWidget( view );
-    view->show();
-    view->setFocus();
+    if ( mStartupCompleted ) {
+      mPartsStack->raiseWidget( view );
+      view->show();
+    }
+
+    if ( mFocusWidgets.contains( plugin->identifier() ) ) {
+      focusWidget = mFocusWidgets[ plugin->identifier() ];
+      if ( focusWidget )
+        focusWidget->setFocus();
+    } else
+      view->setFocus();
+
     mCurrentPlugin = plugin;
     KAction *action = plugin->newActions()->first();
+    
+    createGUI( plugin->part() );
+
     setCaption( i18n( "Plugin dependent window title" ,"%1 - Kontact" ).arg( plugin->title() ) );
+
     if ( action ) {
       mNewActions->setIcon( action->icon() );
       mNewActions->setText( action->text() );
@@ -575,8 +585,12 @@ void MainWindow::selectPlugin( Kontact::Plugin *plugin )
   QStringList::ConstIterator it;
   for ( it = invisibleActions.begin(); it != invisibleActions.end(); ++it ) {
     KAction *action = part->actionCollection()->action( (*it).latin1() );
-    if ( action )
-      action->unplug( toolBar() );
+    if ( action ) {
+      QPtrListIterator<KToolBar> it(  toolBarIterator() );
+      for (  ; it.current() ; ++it ) {
+        action->unplug( it.current() );
+      }
+    }
   }
 
   KApplication::restoreOverrideCursor();
@@ -625,35 +639,38 @@ void MainWindow::slotRequestFeature()
     kapp->invokeBrowser( "http://kontact.org/shopping" );
 }
 
-void MainWindow::showTip(bool force)
+void MainWindow::slotShowIntroduction()
+{
+  mPartsStack->raiseWidget( 0 ); // ###
+}
+
+void MainWindow::showTip( bool force )
 {
   QStringList tips;
   PluginList::ConstIterator end = mPlugins.end();
   for ( PluginList::ConstIterator it = mPlugins.begin(); it != end; ++it ) {
-    QString file = ( *it )->tipFile();
+    QString file = (*it)->tipFile();
     if ( !file.isEmpty() )
       tips.append( file );
   }
 
-  KTipDialog::showMultiTip(this, tips, force);
+  KTipDialog::showMultiTip( this, tips, force );
 }
 
 void MainWindow::slotQuit()
 {
-  mReallyClose=true;
+  mReallyClose = true;
   close();
 }
 
 void MainWindow::slotPreferences()
 {
   static KSettings::Dialog *dlg = 0;
-  if( !dlg ) {
+  if ( !dlg ) {
     dlg = new KSettings::Dialog( KSettings::Dialog::Configurable, this );
     dlg->addPluginInfos( mPluginInfos );
     connect( dlg, SIGNAL( pluginSelectionChanged() ),
-         SLOT( pluginsChanged() ) );
-//    connect( dlg, SIGNAL( okClicked() ), SLOT( pluginsChanged() ) );
-//    connect( dlg, SIGNAL( applyClicked() ), SLOT( pluginsChanged() ) );
+             SLOT( pluginsChanged() ) );
   }
 
   dlg->show();
@@ -680,45 +697,29 @@ int MainWindow::startServiceFor( const QString& serviceType,
 {
   PluginList::ConstIterator end = mPlugins.end();
   for ( PluginList::ConstIterator it = mPlugins.begin(); it != end; ++it ) {
-    if ( ( *it )->createDCOPInterface( serviceType ) ) {
+    if ( (*it)->createDCOPInterface( serviceType ) ) {
       kdDebug(5600) << "found interface for " << serviceType << endl;
       if ( dcopService )
-        *dcopService = ( *it )->dcopClient()->appId();
-      kdDebug(5600) << "appId=" << ( *it )->dcopClient()->appId() << endl;
+        *dcopService = (*it)->dcopClient()->appId();
+      kdDebug(5600) << "appId=" << (*it)->dcopClient()->appId() << endl;
       return 0; // success
     }
   }
+
   kdDebug(5600) <<
     "Didn't find dcop interface, falling back to external process" << endl;
+
   return KDCOPServiceStarter::startServiceFor( serviceType, constraint,
       preferences, error, dcopService, flags );
 }
 
-void MainWindow::setHeaderText( const QString &text )
-{
-  mInfoExtCache[ mLastInfoExtension ].text = text;
-  mHeaderText->setText( text );
-}
-
-void MainWindow::setHeaderPixmap( const QPixmap &pixmap )
-{
-  QPixmap pm( pixmap );
-
-  if ( pm.height() > 22 || pm.width() > 22 ) {
-    QImage img;
-    img = pixmap;
-    pm = img.smoothScale( 22, 22, QImage::ScaleMin );
-  }
-
-  mInfoExtCache[ mLastInfoExtension ].pixmap = pm;
-  mHeaderPixmap->setPixmap( pm );
-}
-
 void MainWindow::pluginsChanged()
 {
+  unplugActionList( "navigator_actionlist" );
   unloadPlugins();
   loadPlugins();
   mSidePane->updatePlugins();
+  plugActionList( "navigator_actionlist", mSidePane->actions() );
 }
 
 void MainWindow::updateConfig()
@@ -726,45 +727,6 @@ void MainWindow::updateConfig()
   kdDebug( 5600 ) << k_funcinfo << endl;
 
   saveSettings();
-
-#if 0
-  bool sidePaneChanged = ( Prefs::self()->mSidePaneType != mSidePaneType );
-
-  if ( sidePaneChanged ) {
-    mSidePaneType = Prefs::self()->mSidePaneType;
-
-    delete mSidePane;
-
-    switch ( mSidePaneType ) {
-      case Prefs::SidePaneIcons:
-        mSidePane = new IconSidePane( this, mSplitter );
-        mHeaderFrame->hide();
-        break;
-      default:
-        kdError() << "Invalid SidePaneType: " << mSidePaneType << endl;
-      case Prefs::SidePaneBars:
-        mSidePane = new SidePane( this, mSplitter );
-        mHeaderFrame->show();
-        break;
-    }
-
-    mSplitter->setResizeMode( mSidePane, QSplitter::KeepSize );
-
-    mSidePane->setSizePolicy( QSizePolicy( QSizePolicy::Maximum,
-                               QSizePolicy::Preferred ) );
-
-    connect( mSidePane, SIGNAL( pluginSelected( Kontact::Plugin * ) ),
-             SLOT( selectPlugin( Kontact::Plugin * ) ) );
-
-    mSplitter->moveToFirst( mSidePane );
-
-    mSidePane->show();
-  }
-
-  if ( sidePaneChanged )
-    mSidePane->updatePlugins();
-#endif
-
   loadSettings();
 }
 
@@ -807,6 +769,20 @@ void MainWindow::slotNewToolbarConfig()
   applyMainWindowSettings( KGlobal::config(), "MainWindow" );
 }
 
+void MainWindow::slotOpenUrl( const KURL &url )
+{
+  if ( url.protocol() == "exec" ) {
+    if ( url.path() == "/switch" ) {
+      selectPlugin( mCurrentPlugin );
+    }
+    if ( url.path() == "/gwwizard" ) {
+      KRun::runCommand( "groupwarewizard" );
+      slotQuit();
+    }
+  } else
+    new KRun( url, this );
+}
+
 bool MainWindow::queryClose()
 {
   if ( kapp->sessionSaving() || mReallyClose )
@@ -825,13 +801,60 @@ bool MainWindow::queryClose()
   return localClose;
 }
 
-
 void MainWindow::slotShowStatusMsg( const QString &msg )
 {
-  if ( !statusBar() || !mStatusMsgLabel ) return;
-  int statusWidth = mStatusMsgLabel->width() - fontMetrics().maxWidth();
-  QString text = KStringHandler::rPixelSqueeze( " " + msg, fontMetrics(),
-                                                statusWidth );
-  mStatusMsgLabel->setText( text );
+  if ( !statusBar() || !mStatusMsgLabel )
+     return;
+
+  mStatusMsgLabel->setText( msg );
 }
+
+QString MainWindow::introductionString()
+{
+  KIconLoader *iconloader = KGlobal::iconLoader();
+  int iconSize = iconloader->currentSize( KIcon::Desktop );
+
+  QString handbook_icon_path = iconloader->iconPath( "contents2",  KIcon::Desktop );
+  QString html_icon_path = iconloader->iconPath( "html",  KIcon::Desktop );
+  QString wizard_icon_path = iconloader->iconPath( "wizard",  KIcon::Desktop );
+
+  QString info = i18n( "<h2 style='text-align:center; margin-top: 0px;'>Welcome to Kontact %1</h2>"
+      "<p>%1</p>"
+      "<table align=\"center\">"
+      "<tr><td><a href=\"%1\"><img width=\"%1\" height=\"%1\" src=\"%1\" /></a></td>"
+      "<td><a href=\"%1\">%1</a><br><span id=\"subtext\"><nobr>%1</td></tr>"
+      "<tr><td><a href=\"%1\"><img width=\"%1\" height=\"%1\" src=\"%1\" /></a></td>"
+      "<td><a href=\"%1\">%1</a><br><span id=\"subtext\"><nobr>%1</td></tr>"
+      "<tr><td><a href=\"%1\"><img width=\"%1\" height=\"%1\" src=\"%1\" /></a></td>"
+      "<td><a href=\"%1\">%1</a><br><span id=\"subtext\"><nobr>%1</td></tr>"
+      "</table>"
+      "<p style=\"margin-bottom: 0px\"> <a href=\"%1\">Skip this introduction</a></p>" )
+      .arg( kapp->aboutData()->version() )
+      .arg( i18n( "Kontact handles your e-mail, addressbook, calendar, to-do list and more." ) )
+      .arg( "help:/kontact" )
+      .arg( iconSize )
+      .arg( iconSize )
+      .arg( handbook_icon_path )
+      .arg( "help:/kontact" )
+      .arg( i18n( "Read Manual" ) )
+      .arg( i18n( "Learn more about Kontact and its components" ) )
+      .arg( "http://kontact.org" )
+      .arg( iconSize )
+      .arg( iconSize )
+      .arg( html_icon_path )
+      .arg( "http://kontact.org" )
+      .arg( i18n( "Visit Kontact Website" ) )
+      .arg( i18n( "Access online resources and tutorials" ) )
+      .arg( "exec:/gwwizard" )
+      .arg( iconSize )
+      .arg( iconSize )
+      .arg( wizard_icon_path )
+      .arg( "exec:/gwwizard" )
+      .arg( i18n( "Configure Kontact as Groupware Client" ) )
+      .arg( i18n( "Prepare Kontact for use in corporate networks" ) )
+      .arg( "exec:/switch" );
+  return info;
+}
+
+
 #include "mainwindow.moc"
