@@ -320,6 +320,10 @@ KPIM::EmailParseResult KPIM::isValidEmailAddress( const QString& aStr )
             commentLevel = 1;
           }
           break;
+        case ':' :
+          if ( !inQuotedString ) {
+            return DisallowedChar;
+          }
         case '<' :
           if ( !inQuotedString ) {
             context = InAngleAddress;
@@ -380,6 +384,11 @@ KPIM::EmailParseResult KPIM::isValidEmailAddress( const QString& aStr )
 
     case InAngleAddress : {
       switch ( aStr[index] ) {
+        case ',' :
+          if ( !inQuotedString ) {
+            return UnexpectedComma;
+          }
+          break;
         case '"' : inQuotedString = !inQuotedString;
             break;
         case '@' :
@@ -466,6 +475,9 @@ QString KPIM::emailParseResultToString( EmailParseResult errorCode )
       return i18n("The email address you entered is not valid because it "
                   "does not seem to contain an actual email address, i.e. "
                   "something of the form joe@kde.org.");
+    case DisallowedChar :
+      return i18n("The email address you entered is not valid because it "
+                  "contains an illegal character.");
   }
   return i18n("Unknown problem with email address");
 }
@@ -502,7 +514,7 @@ bool KPIM::isValidSimpleEmailAddress( const QString& aStr )
       }
   }
 
-  QString addrRx = "[a-zA-Z]*[\\w.-]*[a-zA-Z0-9]@";
+  QString addrRx = "[a-zA-Z]*[~|{}`\\^?=/+*'&%$#!_\\w.-]*[~|{}`\\^?=/+*'&%$#!_a-zA-Z0-9-]@";
   if ( localPart[ 0 ] == '\"' || localPart[ localPart.length()-1 ] == '\"' ) {
     addrRx = "\"[a-zA-Z@]*[\\w.@-]*[a-zA-Z0-9@]\"@";
   }
@@ -685,14 +697,24 @@ bool KPIM::getNameAndMail(const QString& aStr, QString& name, QString& mail)
     //          and outside of quoted text behind the trailing '>'.
     bInComment = false;
     bInQuotesOutsideOfEmail = false;
+    int parenthesesNesting = 0;
     for( i = iAd+1; len > i; ++i ) {
       c = aStr[i];
       if( bInComment ){
         if( ')' == c ){
-          if( !name.isEmpty() )
-            name.append( ' ' );
-          bInComment = false;
-        }else{
+          if ( --parenthesesNesting == 0 ) {
+            bInComment = false;
+            if( !name.isEmpty() )
+              name.append( ' ' );
+          } else {
+            // nested ")", add it
+            name.append( ')' ); // name can't be empty here
+          }
+        } else {
+          if( '(' == c ) {
+            // nested "("
+            ++parenthesesNesting;
+          }
           name.append( c ); // all comment stuff is part of the name
         }
       }else if( bInQuotesOutsideOfEmail ){
@@ -718,7 +740,8 @@ bool KPIM::getNameAndMail(const QString& aStr, QString& name, QString& mail)
             case '(':
               if( !name.isEmpty() )
                 name.append( ' ' );
-              bInComment = true;
+              if ( ++parenthesesNesting > 0 )
+                bInComment = true;
               break;
             default:
               if( ' ' != c )
@@ -759,8 +782,10 @@ QString KPIM::normalizedAddress( const QString & displayName,
     return addrSpec;
   else if ( comment.isEmpty() )
     return displayName + " <" + addrSpec + ">";
-  else if ( displayName.isEmpty() )
-    return comment + " <" + addrSpec + ">";
+  else if ( displayName.isEmpty() ) {
+    QString commentStr = comment;
+    return quoteNameIfNecessary( commentStr ) + " <" + addrSpec + ">";
+  }
   else
     return displayName + " (" + comment + ") <" + addrSpec + ">";
 }
@@ -771,7 +796,7 @@ QString KPIM::decodeIDN( const QString & addrSpec )
 {
   const int atPos = addrSpec.findRev( '@' );
   if ( atPos == -1 )
-    return QString::null;
+    return addrSpec;
 
   QString idn = KIDNA::toUnicode( addrSpec.mid( atPos + 1 ) );
   if ( idn.isEmpty() )
@@ -873,3 +898,50 @@ QString KPIM::normalizeAddressesAndEncodeIDNs( const QString & str )
   return normalizedAddressList.join( ", " );
 }
 
+
+//-----------------------------------------------------------------------------
+// Escapes unescaped doublequotes in str.
+static QString escapeQuotes( const QString & str )
+{
+  if ( str.isEmpty() )
+    return QString();
+
+  QString escaped;
+  // reserve enough memory for the worst case ( """..."" -> \"\"\"...\"\" )
+  escaped.reserve( 2*str.length() );
+  unsigned int len = 0;
+  for ( unsigned int i = 0; i < str.length(); ++i, ++len ) {
+    if ( str[i] == '"' ) { // unescaped doublequote
+      escaped[len] = '\\';
+      ++len;
+    }
+    else if ( str[i] == '\\' ) { // escaped character
+      escaped[len] = '\\';
+      ++len;
+      ++i;
+      if ( i >= str.length() ) // handle trailing '\' gracefully
+        break;
+    }
+    escaped[len] = str[i];
+  }
+  escaped.truncate( len );
+  return escaped;
+}
+
+//-----------------------------------------------------------------------------
+QString KPIM::quoteNameIfNecessary( const QString &str )
+{
+  QString quoted = str;
+
+  QRegExp needQuotes(  "[^ 0-9A-Za-z\\x0080-\\xFFFF]" );
+  // avoid double quoting
+  if ( ( quoted[0] == '"' ) && ( quoted[quoted.length() - 1] == '"' ) ) {
+    quoted = "\"" + escapeQuotes( quoted.mid( 1, quoted.length() - 2 ) ) + "\"";
+  }
+  else if ( quoted.find( needQuotes ) != -1 ) {
+    quoted = "\"" + escapeQuotes( quoted ) + "\"";
+  }
+
+  return quoted;
+}
+
