@@ -196,9 +196,14 @@ void AddresseeLineEdit::insert( const QString &t )
   if ( newText.isEmpty() )
     return;
 
-  // remove newlines in the to-be-pasted string as well as an eventual
-  // mailto: protocol
-  newText.replace( QRegExp("\r?\n"), ", " );
+  // remove newlines in the to-be-pasted string
+  QStringList lines = QStringList::split( QRegExp("\r?\n"), newText, false );
+  for ( QStringList::iterator it = lines.begin();
+       it != lines.end(); ++it ) {
+    // remove trailing commas and whitespace
+    (*it).remove( QRegExp(",?\\s*$") );
+  }
+  newText = lines.join( ", " );
 
   if ( newText.startsWith("mailto:") ) {
     KURL url( newText );
@@ -503,6 +508,8 @@ void AddresseeLineEdit::addContact( const KABC::Addressee& addr, int weight )
     if ( !name.isEmpty() )
       addCompletionItem( addr.preferredEmail() + " (" + name + ")", weight );
 
+    if ( name.find( ',' ) != -1 ) return; // probably already of the form "Lastname, Firstname"
+
     bool bDone = false;
     int i = 1;
     do{
@@ -538,7 +545,7 @@ void AddresseeLineEdit::addCompletionItem( const QString& string, int weight )
 {
   // Check if there is an exact match for item already, and use the max weight if so.
   // Since there's no way to get the information from KCompletion, we have to keep our own QMap
-  CompletionItemsMap::iterator it = s_completionItemMap->begin();
+  CompletionItemsMap::iterator it = s_completionItemMap->find( string );
   if ( it != s_completionItemMap->end() ) {
     weight = QMAX( *it, weight );
     *it = weight;
